#include <math.h>    
#include "trigfun.h"  
/* 
 *  
 *            Syntax  [sys, x0] = synchgenerator(t,x,u,flag,X0,Xd,Xq,Lfd,Lf,Rf,omegaS) 
*/ 
/* 
* The following #define is used to specify the name of this S-Function. 
*/ 
#define S_FUNCTION_NAME synchgenerator 
/* 
* need to include simstruc.h for the definition of the SimStruct and 
* its associated macro definitions. 
*/ 
#include "simstruc.h" 
  
/* 
 * mdlInitializeSizes - initialize the sizes array 
 * 
 * The sizes array is used by SIMULINK to determine the S-function block's 
 * characteristics (number of inputs, outputs, states, etc.). 
 */ 
static void mdlInitializeSizes(S) 
    SimStruct *S; 
{ 
    ssSetNumContStates(    S, 2);      /* number of continuous states */ 
    ssSetNumDiscStates(    S, 0);      /* number of discrete states */ 
    ssSetNumInputs(        S, 4);      /* number of inputs */ 
    ssSetNumOutputs(       S, 9);      /* number of outputs */ 
    ssSetDirectFeedThrough(S, 1);      /* direct feedthrough flag */ 
    ssSetNumSampleTimes(   S, 1);      /* number of sample times */ 
    ssSetNumInputArgs(     S, 7);      /* number of input arguments */ 
    ssSetNumRWork(         S, 0);      /* number of real work vector elements */ 
    ssSetNumIWork(         S, 0);      /* number of integer work vector elements */ 
    ssSetNumPWork(         S, 0);      /* number of pointer work vector elements */ 
     /* 
     * if there aren't the correct number of parameters, just return, simulink.c will 
     * flag the error 
     */ 
    if (ssGetNumArgs(S) != 7) 
       return; 
    if (mxGetM(ssGetArg(S,0)) != 2  || mxGetN(ssGetArg(S,0)) !=1)  { 
        ssSetErrorStatus( S,"X0 must have dimensions: 2 by 1"); 
        return; 
    } 
    if (mxGetM(ssGetArg(S,1)) != 1  || mxGetN(ssGetArg(S,1)) !=1)  { 
        ssSetErrorStatus( S,"Xd must have dimensions: 1 by 1"); 
        return; 
    } 
    if (mxGetM(ssGetArg(S,2)) != 1  || mxGetN(ssGetArg(S,2)) !=1)  { 
        ssSetErrorStatus( S,"Xq must have dimensions: 1 by 1"); 
        return; 
    } 
    if (mxGetM(ssGetArg(S,3)) != 1  || mxGetN(ssGetArg(S,3)) !=1)  { 
        ssSetErrorStatus( S,"Lfd must have dimensions: 1 by 1"); 
        return; 
    } 
    if (mxGetM(ssGetArg(S,4)) != 1  || mxGetN(ssGetArg(S,4)) !=1)  { 
        ssSetErrorStatus( S,"Lf must have dimensions: 1 by 1"); 
        return; 
    } 
    if (mxGetM(ssGetArg(S,5)) != 1  || mxGetN(ssGetArg(S,5)) !=1)  { 
        ssSetErrorStatus( S,"Rf must have dimensions: 1 by 1"); 
        return; 
    } 
    if (mxGetM(ssGetArg(S,6)) != 1  || mxGetN(ssGetArg(S,6)) !=1)  { 
        ssSetErrorStatus( S,"omegaS must have dimensions: 1 by 1"); 
        return; 
    } 
} 
/* 
 * mdlInitializeSampleTimes - initialize the sample times array 
 * 
 * This function is used to specify the sample time(s) for your S-function. 
 * If your S-function is continuous, you must specify a sample time of 0.0. 
 * Sample times must be registered in ascending order. 
 */ 
static void mdlInitializeSampleTimes(S) 
    SimStruct *S; 
{ 
    ssSetSampleTimeEvent(S, 0, 0.0); 
    ssSetOffsetTimeEvent(S, 0, 0.0); 
} 
/* 
 * mdlInitializeConditions - initialize the states 
 * 
 * In this function, you should initialize the continuous and discrete 
 * states for your S-function block.  The initial states are placed 
 * in the x0 variable.  You can also perform any other initialization 
 * activities that your S-function may require. 
 */ 
static void mdlInitializeConditions(x0, S) 
#ifdef MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */ 
    double *x0; 
    SimStruct *S; 
{ 
    int i,imax; 
    double *X0pr; 
/* Set initial value for state vector */ 
    X0pr = mxGetPr(ssGetArg(S,0));  
    imax = mxGetM(ssGetArg(S,0)); 
    for(i=0;i<imax;i++){ 
        x0[i] = X0pr[i]; 
    } 
} 
#else    /* compiled for dSpace board? */ 
    real_T *x0; 
    SimStruct *S; 
{ 
    int_T i,imax; 
    real_T *X0pr; 
/* Set initial value for state vector */ 
    X0pr = mxGetPr(ssGetArg(S,0));  
    imax = mxGetM(ssGetArg(S,0)); 
    for(i=0;i<imax;i++){ 
        x0[i] = X0pr[i]; 
    } 
} 
#endif 
/* 
 * mdlDerivatives - compute the derivatives 
 * 
 * In this function, you compute the S-function block's derivatives. 
 * The derivatives are placed in the dx variable. 
 */ 
static void mdlDerivatives(dx, x, u, S, tid) 
#ifdef MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */ 
    double *dx, *x, *u; 
    SimStruct *S; 
    int tid; 
{  
    int n; 
    double s1, s2, s3, s4, s5, s6, s7, s8, s9, s10; 
    double t1, t2, t3, t4, t5; 
    double *RWork = ssGetRWork(S); 
    int i,j; 
    char s='L'; 
    int nrhs, LDA, info; 
    double *Xdpr = mxGetPr(ssGetArg(S,1)); 
    double *Xqpr = mxGetPr(ssGetArg(S,2)); 
    double *Lfdpr = mxGetPr(ssGetArg(S,3)); 
    double *Lfpr = mxGetPr(ssGetArg(S,4)); 
    double *Rfpr = mxGetPr(ssGetArg(S,5)); 
    double *omegaSpr = mxGetPr(ssGetArg(S,6)); 
#else    /* compiled for dSpace board? */ 
    real_T *dx; 
	 const real_T *x, *u; 
    SimStruct *S; 
    int_T tid; 
{  
    int_T n; 
    real_T s1, s2, s3, s4, s5, s6, s7, s8, s9, s10; 
    real_T t1, t2, t3, t4, t5; 
    real_T *RWork = ssGetRWork(S); 
    int_T i,j; 
    char s='L'; 
    int_T nrhs, LDA, info; 
    real_T *Xdpr = mxGetPr(ssGetArg(S,1)); 
    real_T *Xqpr = mxGetPr(ssGetArg(S,2)); 
    real_T *Lfdpr = mxGetPr(ssGetArg(S,3)); 
    real_T *Lfpr = mxGetPr(ssGetArg(S,4)); 
    real_T *Rfpr = mxGetPr(ssGetArg(S,5)); 
    real_T *omegaSpr = mxGetPr(ssGetArg(S,6)); 
#endif 
    n = 2; 
    t1 =    pow(Lfdpr[0],2.); 
    t2 =    1/Lfpr[0]; 
    t3 =    2.4494897; 
    t4 =    cos(u[3]-x[0]); 
    t5 =    1/(t1*t3*omegaSpr[0]+3.*Lfpr[0]*Xdpr[0]); 
  dx[0] =     -omegaSpr[0]+u[0]; 
  dx[1] =     t5*Rfpr[0]*(-t1*t2*t3*t4*omegaSpr[0]*u[2]-3.*x[1]*Xdpr[0]+u[1]*(t1*t2*t3*omegaSpr[0]+3.*Xdpr[0])); 
} 
/* 
 * mdlOutputs - compute the outputs 
 * 
 * In this function, you compute the outputs of your S-function 
 * block.  The outputs are placed in the y variable. 
 */ 
static void mdlOutputs(y, x, u, S, tid) 
#ifdef MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */ 
    double *y, *x, *u; 
    SimStruct *S; 
    int tid; 
{ 
    double s1, s2, s3, s4, s5, s6, s7, s8, s9, s10; 
    double t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11, t12, t13, t14, t15, t16, t17; 
    double *Xdpr = mxGetPr(ssGetArg(S,1)); 
    double *Xqpr = mxGetPr(ssGetArg(S,2)); 
    double *Lfdpr = mxGetPr(ssGetArg(S,3)); 
    double *Lfpr = mxGetPr(ssGetArg(S,4)); 
    double *Rfpr = mxGetPr(ssGetArg(S,5)); 
    double *omegaSpr = mxGetPr(ssGetArg(S,6)); 
#else    /* compiled for dSpace board? */ 
    real_T *y; 
	  const real_T *x, *u; 
    SimStruct *S; 
    int_T tid; 
{ 
    real_T s1, s2, s3, s4, s5, s6, s7, s8, s9, s10; 
    real_T t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11, t12, t13, t14, t15, t16, t17; 
    real_T *Xdpr = mxGetPr(ssGetArg(S,1)); 
    real_T *Xqpr = mxGetPr(ssGetArg(S,2)); 
    real_T *Lfdpr = mxGetPr(ssGetArg(S,3)); 
    real_T *Lfpr = mxGetPr(ssGetArg(S,4)); 
    real_T *Rfpr = mxGetPr(ssGetArg(S,5)); 
    real_T *omegaSpr = mxGetPr(ssGetArg(S,6)); 
#endif 
    t1 =    1/Xqpr[0]; 
    t2 =    pow(Xdpr[0],2.); 
    t3 =    pow(u[2],2.); 
    t4 =    pow(omegaSpr[0],2.); 
    t5 =    pow(Lfdpr[0],3.); 
    t6 =    pow(Lfdpr[0],2.); 
    t7 =    pow(Lfpr[0],2.); 
    t8 =    pow(x[1],2.); 
    t9 =    2.4494897; 
    t10 =    cos(u[3]-x[0]); 
    t11 =    -sin(u[3]-x[0]); 
    t12 =    pow(t11,2.); 
    t13 =    pow(t10,2.); 
    t14 =    1/(2.*t4*t5*t9+6.*Lfdpr[0]*Lfpr[0]*omegaSpr[0]*Xdpr[0]); 
    t15 =    1/(t6*t9*omegaSpr[0]+3.*Lfpr[0]*Xdpr[0]); 
    t16 =    pow(t6*t9*omegaSpr[0]+3.*Lfpr[0]*Xdpr[0],-2.); 
    t17 =    cos(2.*(-u[3]+x[0])); 
    y[0] =     x[0]; 
    y[1] =     t15*(t10*t6*t9*omegaSpr[0]*u[2]+3.*Lfpr[0]*x[1]*Xdpr[0]); 
    y[2] =     t15*Lfpr[0]*(-3.*t10*u[2]+3.*x[1]); 
    y[3] =     -t1*t11*u[2]; 
    y[4] =     t14*(6.*t10*t6*omegaSpr[0]*u[2]+3.*t9*Lfpr[0]*x[1]*Xdpr[0]); 
    y[5] =     t1*t11*t15*(t10*t3*t6*t9*omegaSpr[0]+3.*Lfpr[0]*u[2]*x[1]*Xdpr[0]); 
    y[6] =     t1*t16*(-9.*t2*t7*t8+3.*t13*t3*t6*t9*Lfpr[0]*omegaSpr[0]*Xdpr[0]+x[1]*(9.*t10*t2*t7*u[2]-3.*t10*t6*t9*Lfpr[0]*omegaSpr[0]*u[2]*Xdpr[0])); 
    y[7] =     t11*t15*(t1*t10*t3*t6*t9*omegaSpr[0]+Lfpr[0]*(3.*u[2]*x[1]+t10*t3*(-3.+3.*t1*Xdpr[0]))); 
    y[8] =     t15*(t1*t12*t3*t6*t9*omegaSpr[0]+Lfpr[0]*(-3.*t10*u[2]*x[1]+t3*(1.5+1.5*t17+t1*(1.5*Xdpr[0]-1.5*t17*Xdpr[0])))); 
} 
/* 
 * mdlUpdate - perform action at major integration time step 
 * 
 * This function is called once for every major integration time step. 
 * Discrete states are typically updated here, but this function is useful 
 * for performing any tasks that should only take place once per integration 
 * step. 
 */ 
static void mdlUpdate(x, u, S, tid) 
#ifdef MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */ 
    double  *x, *u; 
    SimStruct *S; 
    int tid; 
#else    /* compiled for dSpace board? */ 
    real_T *x; 
	  const real_T *u; 
    SimStruct *S; 
    int_T tid; 
#endif 
{ 
} 
/* 
 * mdlTerminate - called when the simulation is terminated. 
 * 
 * In this function, you should perform any actions that are necessary 
 * at the termination of a simulation.  For example, if memory was allocated 
 * in mdlInitializeConditions, this is the place to free it. 
 */ 
static void mdlTerminate(S) 
    SimStruct *S; 
{ 
} 
#ifdef MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */ 
#include "simulink.c"      /* MEX-file interface mechanism */ 
#else 
#include "cg_sfun.h"       /* Code generation registration function */ 
#endif 
