(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    128945,       3755]
NotebookOptionsPosition[    120389,       3467]
NotebookOutlinePosition[    120812,       3483]
CellTagsIndexPosition[    120769,       3480]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Synchronous Generator Models", "Title",
 CellChangeTimes->{{3.5571392997460403`*^9, 3.557139309144578*^9}}],

Cell["Harry G. Kwatny", "Subtitle",
 CellChangeTimes->{{3.5571393284266806`*^9, 3.557139338481256*^9}}],

Cell["Drexel University", "Subsubtitle",
 CellChangeTimes->{{3.5571393520940347`*^9, 3.557139356949312*^9}}],

Cell[CellGroupData[{

Cell["Preliminaries", "Section",
 CellChangeTimes->{{3.5571393862099123`*^9, 3.557139389889123*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"<<", "ProPac`"}]], "Input",
 CellChangeTimes->{{3.5571393942033696`*^9, 3.5571394043708*^9}}],

Cell[BoxData["\<\"\\nThis is ProPac - Development Version 5.0\\n\\nProPac has \
been successfully loaded. It contains several packages\\nfor multibody \
dynamics, linear control and nonlinear control.\\nThe package names are:\\n   \
GeoTools\\n   Dynamics\\n   ControlL\\n   ControlN\\n   DifferentialForms\\n  \
 ObserverTools\\n   NDTools\\n   VSCTools\\n   MatlabInterface\\n   \
MEXTools\\nAppropriate packages will be loaded automatically as \
required.\\nTo obtain a list of functions in a package and links to \
function\\ndeclarations, enter:\\n   ?ProPac`PackageName`*\\nHelp is also \
available in the Help Browser under Add-ons.\"\>"], "Print",
 CellChangeTimes->{3.557151973347823*^9, 3.5575107515664067`*^9, 
  3.5575112940525603`*^9, 3.557662284292035*^9, 3.5577431366017447`*^9, 
  3.557845527781798*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Park Transformation", "Section",
 CellChangeTimes->{{3.6376719295203285`*^9, 3.6376719499934998`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"B", "=", 
   RowBox[{
    SqrtBox[
     FractionBox["2", "3"]], 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Cos", "[", "\[Theta]", "]"}], ",", 
        RowBox[{"Cos", "[", 
         RowBox[{"\[Theta]", "-", 
          FractionBox[
           RowBox[{"2", " ", "\[Pi]"}], "3"]}], "]"}], ",", 
        RowBox[{"Cos", "[", 
         RowBox[{"\[Theta]", "+", 
          FractionBox[
           RowBox[{"2", " ", "\[Pi]"}], "3"]}], "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Sin", "[", "\[Theta]", "]"}], ",", 
        RowBox[{"Sin", "[", 
         RowBox[{"\[Theta]", "-", 
          FractionBox[
           RowBox[{"2", " ", "\[Pi]"}], "3"]}], "]"}], ",", 
        RowBox[{"Sin", "[", 
         RowBox[{"\[Theta]", "+", 
          FractionBox[
           RowBox[{"2", " ", "\[Pi]"}], "3"]}], "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        SqrtBox[
         FractionBox["1", "2"]], ",", 
        SqrtBox[
         FractionBox["1", "2"]], ",", 
        SqrtBox[
         FractionBox["1", "2"]]}], "}"}]}], "}"}]}]}], ";", 
  RowBox[{
   RowBox[{"Simplify", "[", "B", "]"}], "//", "MatrixForm"}]}]], "Input"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       SqrtBox[
        FractionBox["2", "3"]], " ", 
       RowBox[{"Cos", "[", "\[Theta]", "]"}]}], 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{"Cos", "[", "\[Theta]", "]"}], 
         SqrtBox["6"]]}], "+", 
       FractionBox[
        RowBox[{"Sin", "[", "\[Theta]", "]"}], 
        SqrtBox["2"]]}], 
      RowBox[{
       RowBox[{"-", 
        SqrtBox[
         FractionBox["2", "3"]]}], " ", 
       RowBox[{"Sin", "[", 
        RowBox[{
         FractionBox["\[Pi]", "6"], "+", "\[Theta]"}], "]"}]}]},
     {
      RowBox[{
       SqrtBox[
        FractionBox["2", "3"]], " ", 
       RowBox[{"Sin", "[", "\[Theta]", "]"}]}], 
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         RowBox[{
          SqrtBox["3"], " ", 
          RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "+", 
         RowBox[{"Sin", "[", "\[Theta]", "]"}]}], 
        SqrtBox["6"]]}], 
      RowBox[{
       SqrtBox[
        FractionBox["2", "3"]], " ", 
       RowBox[{"Cos", "[", 
        RowBox[{
         FractionBox["\[Pi]", "6"], "+", "\[Theta]"}], "]"}]}]},
     {
      FractionBox["1", 
       SqrtBox["3"]], 
      FractionBox["1", 
       SqrtBox["3"]], 
      FractionBox["1", 
       SqrtBox["3"]]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.635778351855225*^9, 3.636131483297048*^9, 
  3.636377025441683*^9, 3.637672162583659*^9, 3.640694928622684*^9, 
  3.6406950973843365`*^9, 3.6406960034341593`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BI", "=", 
  RowBox[{"Transpose", "[", "B", "]"}]}]], "Input",
 CellChangeTimes->{{3.640696098249583*^9, 3.640696111432337*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SqrtBox[
       FractionBox["2", "3"]], " ", 
      RowBox[{"Cos", "[", "\[Theta]", "]"}]}], ",", 
     RowBox[{
      SqrtBox[
       FractionBox["2", "3"]], " ", 
      RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ",", 
     FractionBox["1", 
      SqrtBox["3"]]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       SqrtBox[
        FractionBox["2", "3"]]}], " ", 
      RowBox[{"Sin", "[", 
       RowBox[{
        FractionBox["\[Pi]", "6"], "-", "\[Theta]"}], "]"}]}], ",", 
     RowBox[{
      RowBox[{"-", 
       SqrtBox[
        FractionBox["2", "3"]]}], " ", 
      RowBox[{"Cos", "[", 
       RowBox[{
        FractionBox["\[Pi]", "6"], "-", "\[Theta]"}], "]"}]}], ",", 
     FractionBox["1", 
      SqrtBox["3"]]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       SqrtBox[
        FractionBox["2", "3"]]}], " ", 
      RowBox[{"Sin", "[", 
       RowBox[{
        FractionBox["\[Pi]", "6"], "+", "\[Theta]"}], "]"}]}], ",", 
     RowBox[{
      SqrtBox[
       FractionBox["2", "3"]], " ", 
      RowBox[{"Cos", "[", 
       RowBox[{
        FractionBox["\[Pi]", "6"], "+", "\[Theta]"}], "]"}]}], ",", 
     FractionBox["1", 
      SqrtBox["3"]]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.6406961123973923`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BI", "//", "MatrixForm"}]], "Input",
 CellChangeTimes->{{3.640696130632435*^9, 3.6406961579960003`*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{
       SqrtBox[
        FractionBox["2", "3"]], " ", 
       RowBox[{"Cos", "[", "\[Theta]", "]"}]}], 
      RowBox[{
       SqrtBox[
        FractionBox["2", "3"]], " ", 
       RowBox[{"Sin", "[", "\[Theta]", "]"}]}], 
      FractionBox["1", 
       SqrtBox["3"]]},
     {
      RowBox[{
       RowBox[{"-", 
        SqrtBox[
         FractionBox["2", "3"]]}], " ", 
       RowBox[{"Sin", "[", 
        RowBox[{
         FractionBox["\[Pi]", "6"], "-", "\[Theta]"}], "]"}]}], 
      RowBox[{
       RowBox[{"-", 
        SqrtBox[
         FractionBox["2", "3"]]}], " ", 
       RowBox[{"Cos", "[", 
        RowBox[{
         FractionBox["\[Pi]", "6"], "-", "\[Theta]"}], "]"}]}], 
      FractionBox["1", 
       SqrtBox["3"]]},
     {
      RowBox[{
       RowBox[{"-", 
        SqrtBox[
         FractionBox["2", "3"]]}], " ", 
       RowBox[{"Sin", "[", 
        RowBox[{
         FractionBox["\[Pi]", "6"], "+", "\[Theta]"}], "]"}]}], 
      RowBox[{
       SqrtBox[
        FractionBox["2", "3"]], " ", 
       RowBox[{"Cos", "[", 
        RowBox[{
         FractionBox["\[Pi]", "6"], "+", "\[Theta]"}], "]"}]}], 
      FractionBox["1", 
       SqrtBox["3"]]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.640696139609948*^9, 3.6406961587250414`*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Classical VBR Model", "Section",
 CellChangeTimes->{{3.5571394349119883`*^9, 3.557139439735264*^9}, {
  3.64069223136841*^9, 3.6406922409679585`*^9}}],

Cell[TextData[{
 "The electrical side is modeled as the classical voltage behind transient \
reactance. Electrical dynamics are ignored. Electrical rotor resistance is \
ignored. Mechanical dynamics are included. \n\n",
 StyleBox["Consider the generator connected at the i-th bus.\n\nGenerator \
differential equations\n\n",
  FontSize->14],
 Cell[BoxData[
  FormBox[
   FractionBox[
    SubscriptBox["d\[Delta]", "i"], "dt"], TraditionalForm]]],
 StyleBox[" = ",
  FontSize->14],
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Omega]", "i"], TraditionalForm]],
  FontSize->14],
 StyleBox[" - ",
  FontSize->14],
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Omega]", "s"], TraditionalForm]]],
 StyleBox["\n",
  FontSize->14],
 Cell[BoxData[
  FormBox[
   FractionBox[
    RowBox[{"2", 
     SubscriptBox["H", "i"]}], 
    SubscriptBox["\[Omega]", "s"]], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   FractionBox[
    SubscriptBox["d\[Omega]", "i"], "dt"], TraditionalForm]],
  FontSize->14],
 StyleBox[" = ",
  FontSize->14],
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "Mi"], TraditionalForm]]],
 StyleBox[" - ",
  FontSize->14],
 Cell[BoxData[
  FormBox[
   FractionBox[
    RowBox[{
     SubscriptBox["E", "i"], " ", 
     SubscriptBox["V", "i"], " ", 
     RowBox[{"sin", "(", 
      RowBox[{
       SubscriptBox["\[Delta]", "i"], "-", 
       SubscriptBox["\[Theta]", "i"]}], ")"}]}], 
    SubsuperscriptBox["X", "di", "'"]], TraditionalForm]]],
 StyleBox[" ",
  FontSize->14],
 Cell[BoxData[
  FormBox[
   RowBox[{"-", 
    RowBox[{
     SubscriptBox["D", "i"], "(", 
     RowBox[{
      SubscriptBox["\[Omega]", "i"], "-", 
      SubscriptBox["\[Omega]", "s"]}], ")"}]}], TraditionalForm]],
  FontSize->14],
 StyleBox["\n\nGenerator algebraic equations\n\n",
  FontSize->14],
 Cell[BoxData[
  FormBox[
   SubscriptBox["P", "Gi"], TraditionalForm]],
  FontSize->14],
 StyleBox[" + ",
  FontSize->14],
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["P", "Li"], "(", 
    SubscriptBox["V", "i"], ")"}], TraditionalForm]],
  FontSize->14],
 StyleBox[" = ",
  FontSize->14],
 Cell[BoxData[
  FormBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"j", "=", "1"}], "n"], 
    RowBox[{
     SubscriptBox["V", "i"], 
     SubscriptBox["V", "j"], 
     RowBox[{"sin", "(", 
      RowBox[{
       SubscriptBox["\[Theta]", "i"], "-", 
       SubscriptBox["\[Theta]", "j"]}], ")"}]}]}], TraditionalForm]],
  FontSize->14],
 StyleBox["\n",
  FontSize->14],
 Cell[BoxData[
  FormBox[
   SubscriptBox["Q", "Gi"], TraditionalForm]],
  FontSize->14],
 StyleBox[" + ",
  FontSize->14],
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["Q", "Li"], "(", 
     SubscriptBox["V", "i"], ")"}], "="}], TraditionalForm]],
  FontSize->14],
 Cell[BoxData[
  FormBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"j", "=", "1"}], "n"], 
    RowBox[{
     SubscriptBox["V", "i"], 
     SubscriptBox["V", "j"], 
     RowBox[{"cos", "(", 
      RowBox[{
       SubscriptBox["\[Theta]", "i"], "-", 
       SubscriptBox["\[Theta]", "j"]}], ")"}]}]}], TraditionalForm]],
  FontSize->14],
 StyleBox["\n",
  FontSize->14],
 "\n",
 StyleBox["Here, ",
  FontSize->14],
 Cell[BoxData[
  FormBox[
   SubscriptBox["P", "Gi"], TraditionalForm]],
  FontSize->14],
 StyleBox[" and ",
  FontSize->14],
 Cell[BoxData[
  FormBox[
   SubscriptBox["Q", "Gi"], TraditionalForm]],
  FontSize->14],
 StyleBox[" are the real and reactive powers generated ay bus i. ",
  FontSize->14],
 Cell[BoxData[
  FormBox[
   SubscriptBox["P", "Li"], TraditionalForm]],
  FontSize->14],
 StyleBox[" and ",
  FontSize->14],
 Cell[BoxData[
  FormBox[
   SubscriptBox["Q", "Li"], TraditionalForm]],
  FontSize->14],
 StyleBox[" are the real and reactive power loads on bus i. ",
  FontSize->14],
 "\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["P", "Gi"], "=", 
    FractionBox[
     RowBox[{
      SubscriptBox["E", "i"], 
      SubscriptBox["V", "i"], 
      RowBox[{"sin", "(", 
       RowBox[{
        SubscriptBox["\[Delta]", "i"], "-", 
        SubscriptBox["\[Theta]", "i"]}], ")"}]}], 
     SubsuperscriptBox["X", "di", "'"]]}], TraditionalForm]],
  FontSize->14],
 StyleBox["\n",
  FontSize->14],
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Q", "Gi"], "=", 
    FractionBox[
     RowBox[{"-", 
      SuperscriptBox[
       SubscriptBox["V", "i"], "2"]}], 
     SubsuperscriptBox["X", "di", "'"]]}], TraditionalForm]],
  FontSize->14],
 StyleBox["+",
  FontSize->14],
 Cell[BoxData[
  FormBox[
   FractionBox[
    RowBox[{
     SubscriptBox["E", "i"], 
     SubscriptBox["V", "i"], 
     RowBox[{"cos", "(", 
      RowBox[{
       SubscriptBox["\[Delta]", "i"], "-", 
       SubscriptBox["\[Theta]", "i"]}], ")"}]}], 
    SubsuperscriptBox["X", "di", "'"]], TraditionalForm]],
  FontSize->14],
 "\n\n",
 StyleBox["The internal voltage ",
  FontSize->14],
 Cell[BoxData[
  FormBox[
   SubscriptBox["E", "i"], TraditionalForm]],
  FontSize->14],
 StyleBox[" is a constant given by ",
  FontSize->14],
 Cell[BoxData[
  FormBox[
   RowBox[{"E", "=", 
    SubscriptBox["E", "fd"]}], TraditionalForm]],
  FontSize->14],
 StyleBox["(angle \[Delta]). The variables taken as input from the network \
are V and \[Theta]. The variables passed to the network are ",
  FontSize->14],
 Cell[BoxData[
  FormBox[
   SubscriptBox["P", "G"], TraditionalForm]],
  FontSize->14],
 StyleBox[" and ",
  FontSize->14],
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["Q", "G"], "."}], TraditionalForm]],
  FontSize->14],
 " ",
 StyleBox["The generator current at bus i is",
  FontSize->14],
 "\n\n",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["I", "di"], "+", 
      SubscriptBox["jI", "qi"]}], ")"}], 
    SuperscriptBox["e", 
     RowBox[{"j", "(", 
      RowBox[{"\[Delta]", "-", 
       RowBox[{"\[Pi]", "/", "2"}]}], ")"}]]}], TraditionalForm]]],
 " = ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["I", "Di"], "+", 
    SubscriptBox["jI", "Qi"]}], TraditionalForm]]],
 "\n\n",
 Cell[BoxData[
  FormBox[
   SubscriptBox["I", "di"], TraditionalForm]]],
 " = ",
 Cell[BoxData[
  FormBox[
   FractionBox[
    SubscriptBox["E", "i"], 
    SubsuperscriptBox["X", "di", "'"]], TraditionalForm]]],
 " - ",
 Cell[BoxData[
  FormBox[
   FractionBox[
    SubscriptBox["V", "i"], 
    SubsuperscriptBox["X", "di", "'"]], TraditionalForm]]],
 "cos(\[Theta] - \[Delta])\n",
 Cell[BoxData[
  FormBox[
   SubscriptBox["I", "qi"], TraditionalForm]]],
 " = -",
 Cell[BoxData[
  FormBox[
   FractionBox[
    SubscriptBox["V", "i"], 
    SubsuperscriptBox["X", "di", "'"]], TraditionalForm]]],
 "sin(\[Theta] - \[Delta])",
 "\n\nThe inputs to the generator block are the field voltage, the terminal \
bus voltage magnitude and angle, and the mechanical power.\nThe outputs are \
the rotor angle, the angular velocity, the terminal bus real and reactive \
power.\nThe parameters are initial state, rotor inertia, rotor reactance, \
mechanical damping and synchronous speed."
}], "Text",
 CellChangeTimes->{{3.557139464230665*^9, 3.5571398025251074`*^9}, {
   3.5571398928672743`*^9, 3.557139910294573*^9}, {3.557140027705203*^9, 
   3.5571400628207026`*^9}, {3.5571401997580905`*^9, 3.557140287401189*^9}, {
   3.640692055781367*^9, 3.6406920610726695`*^9}, 3.640692179086419*^9}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"stateList", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"delta", ",", "omega"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"inputs", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"Pm", ",", " ", "Ef", ",", " ", "V", ",", " ", "theta"}], "}"}]}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"passedParams", "=", 
   RowBox[{"{", 
    RowBox[{
    "\"\<X0\>\"", ",", "\"\<H\>\"", ",", "\"\<X\>\"", ",", "\"\<Dm\>\"", ",", 
     "\"\<omegaS\>\""}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"passedParamsDimensions", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"2", ",", "1"}], "}"}], "}"}], ",", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{
          RowBox[{"Length", "[", "passedParams", "]"}], "-", "1"}]}], "}"}]}],
       "]"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"outputEquations", "=", 
   RowBox[{"{", 
    RowBox[{"delta", ",", "omega", ",", "P2", ",", "Q2"}], "}"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"stateEquations", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"omega", "-", "omegaS"}], ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"omegaS", "/", 
        RowBox[{"(", 
         RowBox[{"2", "*", "H"}], ")"}]}], ")"}], "*", 
      RowBox[{"(", 
       RowBox[{"Pm", "-", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"Ef", "*", "V", "*", 
            RowBox[{"Sin", "[", 
             RowBox[{"delta", "-", "theta"}], "]"}]}], ")"}], "/", "X"}], 
         ")"}], "-", 
        RowBox[{"Dm", "*", 
         RowBox[{"(", 
          RowBox[{"omega", "-", "omegaS"}], ")"}]}]}], ")"}]}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"P1", " ", "=", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"Ef", "*", "V", "*", 
      RowBox[{"Sin", "[", 
       RowBox[{"delta", "-", "theta"}], "]"}]}], ")"}], "/", "X"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Q1", " ", "=", " ", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"Ef", "^", "2"}], "/", "X"}], ")"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"Ef", "*", "V", "*", 
       RowBox[{"Cos", "[", 
        RowBox[{"delta", "-", "theta"}], "]"}]}], ")"}], "/", "X"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"P2", " ", "=", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"Ef", "*", "V", "*", 
      RowBox[{"Sin", "[", 
       RowBox[{"delta", "-", "theta"}], "]"}]}], ")"}], "/", "X"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Q2", " ", "=", " ", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"V", "^", "2"}], "/", "X"}], ")"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"Ef", "*", "V", "*", 
       RowBox[{"Cos", "[", 
        RowBox[{"delta", "-", "theta"}], "]"}]}], ")"}], "/", "X"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
  "mexFilename", "=", 
   "\"\<C:\\\\Users\\\\Harry\\\\Documents\\\\Magic \
Briefcase\\\\Projects\\\\PowerSystems\\\\Optimal Energy - NAVSEA \
SBIR\\\\Phase II\\\\Option II\\\\Notes\\\\Machine \
Models\\\\synchronousgenerator.c\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CreateControllerMEX", "[", 
   RowBox[{
   "stateList", ",", "inputs", ",", "outputEquations", ",", "stateEquations", 
    ",", "passedParams", ",", "passedParamsDimensions", ",", "mexFilename"}], 
   "]"}], ";"}]}], "Input",
 CellChangeTimes->{{3.5571399783283186`*^9, 3.5571400005435896`*^9}, {
   3.5571405625785427`*^9, 3.5571405769149675`*^9}, 3.5571487485831375`*^9, {
   3.557151519464018*^9, 3.5571515394841633`*^9}, {3.5571515779983664`*^9, 
   3.5571515907670965`*^9}, {3.5571518974327483`*^9, 3.557151905531212*^9}, {
   3.5587789447535777`*^9, 3.558778946449854*^9}, 3.6406871570391088`*^9, {
   3.6406922898437543`*^9, 3.640692293402958*^9}}],

Cell[CellGroupData[{

Cell[BoxData["\<\"  *** MEXTools successfully loaded ***\"\>"], "Print",
 CellChangeTimes->{3.557151977731074*^9, 3.557510752299608*^9, 
  3.5575112952537622`*^9, 3.5576622854221*^9, 3.5577431391193686`*^9}],

Cell[BoxData["\<\"...Generating Header Code\"\>"], "Print",
 CellChangeTimes->{3.557151977731074*^9, 3.557510752299608*^9, 
  3.5575112952537622`*^9, 3.5576622854221*^9, 3.557743139129369*^9}],

Cell[BoxData["\<\"...Generating Initial Condition Function Code\"\>"], "Print",
 CellChangeTimes->{3.557151977731074*^9, 3.557510752299608*^9, 
  3.5575112952537622`*^9, 3.5576622854221*^9, 3.557743139144369*^9}],

Cell[BoxData["\<\"...Generating State Derivative Function Code\"\>"], "Print",
 CellChangeTimes->{3.557151977731074*^9, 3.557510752299608*^9, 
  3.5575112952537622`*^9, 3.5576622854221*^9, 3.5577431391593695`*^9}],

Cell[BoxData["\<\"...Removing Function terms\"\>"], "Print",
 CellChangeTimes->{3.557151977731074*^9, 3.557510752299608*^9, 
  3.5575112952537622`*^9, 3.5576622854221*^9, 3.5577431391718693`*^9}],

Cell[BoxData["\<\"...Generating temp variable declarations\"\>"], "Print",
 CellChangeTimes->{3.557151977731074*^9, 3.557510752299608*^9, 
  3.5575112952537622`*^9, 3.5576622854221*^9, 3.5577431393063736`*^9}],

Cell[BoxData["\<\"...Converting Expressions to C form\"\>"], "Print",
 CellChangeTimes->{3.557151977731074*^9, 3.557510752299608*^9, 
  3.5575112952537622`*^9, 3.5576622854221*^9, 3.557743139409379*^9}],

Cell[BoxData["\<\"...Generating Output Function Code\"\>"], "Print",
 CellChangeTimes->{3.557151977731074*^9, 3.557510752299608*^9, 
  3.5575112952537622`*^9, 3.5576622854221*^9, 3.5577431394758806`*^9}],

Cell[BoxData["\<\"MEX File created with name: \
C:\\\\Users\\\\Harry\\\\Documents\\\\Magic \
Briefcase\\\\Projects\\\\PowerSystems\\\\Optimal Energy - NAVSEA \
SBIR\\\\Phase II\\\\PowerPac\\\\Notebooks\\\\synchronousgenerator.c\"\>"], \
"Print",
 CellChangeTimes->{3.557151977731074*^9, 3.557510752299608*^9, 
  3.5575112952537622`*^9, 3.5576622854221*^9, 3.5577431398803873`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Internal Bus Model", "Section",
 CellChangeTimes->{{3.557476336556263*^9, 3.557476342603609*^9}, {
  3.5574764154985027`*^9, 3.5574764211769123`*^9}, {3.6406926202486525`*^9, 
  3.640692664302172*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"stateList", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"delta", ",", "omega"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"inputs", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"Pm", ",", " ", "Ef", ",", " ", "Pe"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"passedParams", "=", 
   RowBox[{"{", 
    RowBox[{
    "\"\<X0\>\"", ",", "\"\<H\>\"", ",", "\"\<Dm\>\"", ",", 
     "\"\<omegaS\>\""}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"passedParamsDimensions", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"2", ",", "1"}], "}"}], "}"}], ",", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{
          RowBox[{"Length", "[", "passedParams", "]"}], "-", "1"}]}], "}"}]}],
       "]"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"outputEquations", "=", 
   RowBox[{"{", 
    RowBox[{"delta", ",", "omega", ",", "Ef"}], "}"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"stateEquations", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"omega", "-", "omegaS"}], ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"omegaS", "/", 
        RowBox[{"(", 
         RowBox[{"2", "*", "H"}], ")"}]}], ")"}], "*", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"Pm", "-", "Pe"}], ")"}], "-", 
        RowBox[{"Dm", "*", 
         RowBox[{"(", 
          RowBox[{"omega", "-", "omegaS"}], ")"}]}]}], ")"}]}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
  "mexFilename", "=", 
   "\"\<C:\\\\Users\\\\Harry\\\\Documents\\\\Magic \
Briefcase\\\\Projects\\\\PowerSystems\\\\Optimal Energy - NAVSEA \
SBIR\\\\Phase II\\\\Option II\\\\Notes\\\\Machine \
Models\\\\referencegenerator.c\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CreateControllerMEX", "[", 
   RowBox[{
   "stateList", ",", "inputs", ",", "outputEquations", ",", "stateEquations", 
    ",", "passedParams", ",", "passedParamsDimensions", ",", "mexFilename"}], 
   "]"}], ";"}]}], "Input",
 CellChangeTimes->{{3.5571399783283186`*^9, 3.5571400005435896`*^9}, {
   3.5571405625785427`*^9, 3.5571405769149675`*^9}, 3.5571487485831375`*^9, {
   3.557151519464018*^9, 3.5571515394841633`*^9}, {3.5571515779983664`*^9, 
   3.5571515907670965`*^9}, {3.5571518974327483`*^9, 3.557151905531212*^9}, {
   3.557476430146928*^9, 3.557476436256854*^9}, {3.5575104715927153`*^9, 
   3.5575105010455666`*^9}, {3.557510537487231*^9, 3.5575105796853046`*^9}, {
   3.5575106124141626`*^9, 3.5575106250033846`*^9}, {3.557662091439597*^9, 
   3.557662112392288*^9}, {3.557662198319392*^9, 3.5576622149723444`*^9}, {
   3.557662427869401*^9, 3.5576624497562737`*^9}, {3.557743085014781*^9, 
   3.557743086547305*^9}, {3.5587788575609884`*^9, 3.558778859481018*^9}, 
   3.640687185300725*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"  *** MEXTools successfully loaded ***\"\>"], "Print",
 CellChangeTimes->{
  3.557662472213108*^9, {3.557743143071528*^9, 3.5577431657359543`*^9}, 
   3.5578455283745995`*^9}],

Cell[BoxData["\<\"...Generating Header Code\"\>"], "Print",
 CellChangeTimes->{
  3.557662472213108*^9, {3.557743143071528*^9, 3.5577431657359543`*^9}, 
   3.5578455283745995`*^9}],

Cell[BoxData["\<\"...Generating Initial Condition Function Code\"\>"], "Print",
 CellChangeTimes->{
  3.557662472213108*^9, {3.557743143071528*^9, 3.5577431657359543`*^9}, 
   3.557845528390199*^9}],

Cell[BoxData["\<\"...Generating State Derivative Function Code\"\>"], "Print",
 CellChangeTimes->{
  3.557662472213108*^9, {3.557743143071528*^9, 3.5577431657359543`*^9}, 
   3.557845528390199*^9}],

Cell[BoxData["\<\"...Removing Function terms\"\>"], "Print",
 CellChangeTimes->{
  3.557662472213108*^9, {3.557743143071528*^9, 3.5577431657359543`*^9}, 
   3.557845528405799*^9}],

Cell[BoxData["\<\"...Generating temp variable declarations\"\>"], "Print",
 CellChangeTimes->{
  3.557662472213108*^9, {3.557743143071528*^9, 3.5577431657359543`*^9}, 
   3.557845528405799*^9}],

Cell[BoxData["\<\"...Converting Expressions to C form\"\>"], "Print",
 CellChangeTimes->{
  3.557662472213108*^9, {3.557743143071528*^9, 3.5577431657359543`*^9}, 
   3.557845528421399*^9}],

Cell[BoxData["\<\"...Generating Output Function Code\"\>"], "Print",
 CellChangeTimes->{
  3.557662472213108*^9, {3.557743143071528*^9, 3.5577431657359543`*^9}, 
   3.5578455284681993`*^9}],

Cell[BoxData["\<\"MEX File created with name: \
C:\\\\Users\\\\Harry\\\\Documents\\\\Magic \
Briefcase\\\\Projects\\\\PowerSystems\\\\Optimal Energy - NAVSEA \
SBIR\\\\Phase II\\\\PowerPac\\\\Notebooks\\\\referencegenerator.c\"\>"], \
"Print",
 CellChangeTimes->{
  3.557662472213108*^9, {3.557743143071528*^9, 3.5577431657359543`*^9}, 
   3.5578455285305996`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Balanced Model ", "Section",
 CellChangeTimes->{{3.640692968241556*^9, 3.640693009889939*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"va", "=", 
   RowBox[{
    SqrtBox["2"], "Va", " ", 
    RowBox[{"Cos", "[", "\[Theta]", "]"}]}]}], ";", 
  RowBox[{"vb", "=", 
   RowBox[{
    SqrtBox["2"], " ", "Va", " ", 
    RowBox[{"Cos", "[", 
     RowBox[{"\[Theta]", "-", 
      RowBox[{"2", " ", 
       RowBox[{"\[Pi]", "/", "3"}]}]}], "]"}]}]}], ";", 
  RowBox[{"vc", "=", 
   RowBox[{
    SqrtBox["2"], " ", "Va", " ", 
    RowBox[{"Cos", "[", 
     RowBox[{"\[Theta]", " ", "-", 
      RowBox[{"2", " ", "2", " ", 
       RowBox[{"\[Pi]", "/", "3"}]}]}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ea", "=", 
   RowBox[{
    SqrtBox["2"], " ", "Ea", " ", 
    RowBox[{"Cos", "[", "\[Delta]", "]"}]}]}], ";", 
  RowBox[{"eb", "=", 
   RowBox[{
    SqrtBox["2"], " ", "Ea", " ", 
    RowBox[{"Cos", "[", 
     RowBox[{"\[Delta]", "-", 
      RowBox[{"2", " ", 
       RowBox[{"\[Pi]", "/", "3"}]}]}], "]"}]}]}], ";", 
  RowBox[{"ec", "=", 
   RowBox[{
    SqrtBox["2"], " ", "Ea", " ", 
    RowBox[{"Cos", "[", 
     RowBox[{"\[Delta]", " ", "-", 
      RowBox[{"2", " ", "2", " ", 
       RowBox[{"\[Pi]", "/", "3"}]}]}], "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.6406955088038683`*^9, 3.6406956280636897`*^9}, {
  3.640695683977888*^9, 3.6406957205299788`*^9}, {3.640695791754052*^9, 
  3.640695799568499*^9}, {3.6406958311623063`*^9, 3.6406958318873477`*^9}, {
  3.6406964972404037`*^9, 3.6406965077000017`*^9}, {3.6410225874247913`*^9, 
  3.6410225997904987`*^9}, {3.6410476808459506`*^9, 3.6410477974126177`*^9}, {
  3.6410549804001827`*^9, 3.6410549991682563`*^9}, {3.641055050737206*^9, 
  3.6410550890423965`*^9}, {3.641055127875618*^9, 3.6410551428824763`*^9}, {
  3.6410561405505395`*^9, 3.6410561602426662`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"ia", ",", "ib", ",", "ic"}], "}"}], "=", 
  RowBox[{"BI", ".", 
   RowBox[{"{", 
    RowBox[{"id", ",", "iq", ",", "0"}], "}"}]}]}]], "Input",
 CellChangeTimes->{{3.6406951164444265`*^9, 3.640695146063121*^9}, 
   3.6406959736884584`*^9, {3.6406960479907084`*^9, 3.6406960594513636`*^9}, {
   3.6406961993373647`*^9, 3.6406962000174036`*^9}, {3.64105524679342*^9, 
   3.641055254329851*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{
     SqrtBox[
      FractionBox["2", "3"]], " ", "id", " ", 
     RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "+", 
    RowBox[{
     SqrtBox[
      FractionBox["2", "3"]], " ", "iq", " ", 
     RowBox[{"Sin", "[", "\[Theta]", "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"-", 
      SqrtBox[
       FractionBox["2", "3"]]}], " ", "iq", " ", 
     RowBox[{"Cos", "[", 
      RowBox[{
       FractionBox["\[Pi]", "6"], "-", "\[Theta]"}], "]"}]}], "-", 
    RowBox[{
     SqrtBox[
      FractionBox["2", "3"]], " ", "id", " ", 
     RowBox[{"Sin", "[", 
      RowBox[{
       FractionBox["\[Pi]", "6"], "-", "\[Theta]"}], "]"}]}]}], ",", 
   RowBox[{
    RowBox[{
     SqrtBox[
      FractionBox["2", "3"]], " ", "iq", " ", 
     RowBox[{"Cos", "[", 
      RowBox[{
       FractionBox["\[Pi]", "6"], "+", "\[Theta]"}], "]"}]}], "-", 
    RowBox[{
     SqrtBox[
      FractionBox["2", "3"]], " ", "id", " ", 
     RowBox[{"Sin", "[", 
      RowBox[{
       FractionBox["\[Pi]", "6"], "+", "\[Theta]"}], "]"}]}]}]}], 
  "}"}]], "Output",
 CellChangeTimes->{
  3.6406951486172667`*^9, 3.6406959754265575`*^9, {3.6406960494067893`*^9, 
   3.6406960614784794`*^9}, 3.640696201155469*^9, 3.641022608396991*^9, 
   3.6410551063823886`*^9, 3.6410551564852543`*^9, 3.6410552573390226`*^9, 
   3.6410561707392664`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"B", ".", 
   RowBox[{"{", 
    RowBox[{"va", ",", "vb", ",", "vc"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.640696406468212*^9, 3.6406964587682033`*^9}, {
   3.641022704420483*^9, 3.6410227134019966`*^9}, 3.641055164082689*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SqrtBox["3"], " ", "Va"}], ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{{3.6406964342988033`*^9, 3.6406964619233837`*^9}, 
   3.6410225684037037`*^9, 3.6410226289351654`*^9, 3.6410227160691495`*^9, 
   3.64105516480473*^9, 3.64105618164089*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"B", ".", 
   RowBox[{"{", 
    RowBox[{"ea", ",", "eb", ",", "ec"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.641047858236097*^9, 3.641047867579631*^9}, 
   3.6410551812346697`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    SqrtBox["3"], " ", "Ea", " ", 
    RowBox[{"Cos", "[", 
     RowBox[{"\[Delta]", "-", "\[Theta]"}], "]"}]}], ",", 
   RowBox[{
    RowBox[{"-", 
     SqrtBox["3"]}], " ", "Ea", " ", 
    RowBox[{"Sin", "[", 
     RowBox[{"\[Delta]", "-", "\[Theta]"}], "]"}]}], ",", "0"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.6410478685946894`*^9, 3.641055182027715*^9, 
  3.6410561959727097`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ia", ",", "ib", ",", "ic"}], "}"}], ".", 
   RowBox[{"{", 
    RowBox[{"va", ",", "vb", ",", "vc"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6410553268950014`*^9, 3.641055368366373*^9}}],

Cell[BoxData[
 RowBox[{
  SqrtBox["3"], " ", "id", " ", "Va"}]], "Output",
 CellChangeTimes->{{3.6410553506683607`*^9, 3.641055369026411*^9}, 
   3.641056206380305*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ia", ",", "ib", ",", "ic"}], "}"}], ".", 
   RowBox[{"{", 
    RowBox[{"ea", ",", "eb", ",", "ec"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.6410553268950014`*^9, 3.641055368366373*^9}, {
  3.6410555465095625`*^9, 3.641055555889099*^9}}],

Cell[BoxData[
 RowBox[{
  SqrtBox["3"], " ", "Ea", " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"id", " ", 
     RowBox[{"Cos", "[", 
      RowBox[{"\[Delta]", "-", "\[Theta]"}], "]"}]}], "-", 
    RowBox[{"iq", " ", 
     RowBox[{"Sin", "[", 
      RowBox[{"\[Delta]", "-", "\[Theta]"}], "]"}]}]}], ")"}]}]], "Output",
 CellChangeTimes->{{3.6410553506683607`*^9, 3.641055369026411*^9}, 
   3.6410555577152033`*^9, 3.641056214005741*^9, 3.641057022109962*^9}]
}, Open  ]],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.6410552990864105`*^9, 3.64105531253718*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DifEqns", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", "Ld"}], " ", "did"}], "+", " ", 
       RowBox[{"Lfd", " ", "dif"}]}], "\[Equal]", 
      RowBox[{
       RowBox[{"R", " ", "id"}], " ", "+", " ", 
       RowBox[{"omega", " ", "Lq", " ", "iq"}], " ", "+", "vd"}]}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"-", "Lq"}], " ", "diq"}], "\[Equal]", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "omega"}], " ", "Ld", " ", "id"}], " ", "+", 
       RowBox[{"R", " ", "iq"}], " ", "+", 
       RowBox[{"omega", " ", "Lfd", " ", "if"}], " ", "+", "vq"}]}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"-", " ", "Lfd"}], " ", "did"}], " ", "+", " ", 
       RowBox[{"Lf", " ", "dif"}]}], " ", "\[Equal]", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "Rf"}], " ", "if"}], " ", "+", "vf"}]}]}], "}"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.6406934936416073`*^9, 3.640693583240732*^9}, {
  3.6406936587690525`*^9, 3.6406939378500147`*^9}, {3.6410351237928305`*^9, 
  3.641035124337862*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sols", "=", 
  RowBox[{"Solve", "[", 
   RowBox[{"DifEqns", ",", 
    RowBox[{"{", 
     RowBox[{"did", ",", "diq", ",", "dif"}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6406939475025673`*^9, 3.6406939964183645`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"did", "\[Rule]", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{
        RowBox[{"iq", " ", "Lf", " ", "Lq", " ", "omega"}], "+", 
        RowBox[{"id", " ", "Lf", " ", "R"}], "+", 
        RowBox[{"if", " ", "Lfd", " ", "Rf"}], "+", 
        RowBox[{"Lf", " ", "vd"}], "-", 
        RowBox[{"Lfd", " ", "vf"}]}], 
       RowBox[{
        RowBox[{"Ld", " ", "Lf"}], "-", 
        SuperscriptBox["Lfd", "2"]}]]}]}], ",", 
    RowBox[{"diq", "\[Rule]", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{
        RowBox[{
         RowBox[{"-", "id"}], " ", "Ld", " ", "omega"}], "+", 
        RowBox[{"if", " ", "Lfd", " ", "omega"}], "+", 
        RowBox[{"iq", " ", "R"}], "+", "vq"}], "Lq"]}]}], ",", 
    RowBox[{"dif", "\[Rule]", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{
        RowBox[{"iq", " ", "Lfd", " ", "Lq", " ", "omega"}], "+", 
        RowBox[{"id", " ", "Lfd", " ", "R"}], "+", 
        RowBox[{"if", " ", "Ld", " ", "Rf"}], "+", 
        RowBox[{"Lfd", " ", "vd"}], "-", 
        RowBox[{"Ld", " ", "vf"}]}], 
       RowBox[{
        RowBox[{"Ld", " ", "Lf"}], "-", 
        SuperscriptBox["Lfd", "2"]}]]}]}]}], "}"}], "}"}]], "Output",
 CellChangeTimes->{3.6406939974364233`*^9, 3.641035133656395*^9}]
}, Open  ]],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.6406953282735424`*^9, 3.640695344177452*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Vrules", "=", 
  RowBox[{"Inner", "[", 
   RowBox[{"Rule", ",", 
    RowBox[{"{", 
     RowBox[{"vd", ",", "vq", ",", "v0"}], "}"}], ",", 
    RowBox[{
     RowBox[{"FullSimplify", "[", 
      RowBox[{"B", ".", 
       RowBox[{"{", 
        RowBox[{"va", ",", "vb", ",", "vc"}], "}"}]}], "]"}], "/.", 
     RowBox[{"\[Theta]", "\[Rule]", "theta"}]}], ",", "List"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.641035423413968*^9, 3.641035433784561*^9}, {
  3.641035640626392*^9, 3.6410357716198845`*^9}, {3.6410408605548525`*^9, 
  3.641040878241864*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"vd", "\[Rule]", 
    RowBox[{
     SqrtBox[
      FractionBox["3", "2"]], " ", "Va"}]}], ",", 
   RowBox[{"vq", "\[Rule]", "0"}], ",", 
   RowBox[{"v0", "\[Rule]", "0"}]}], "}"}]], "Output",
 CellChangeTimes->{3.641035775579111*^9, 3.641040879138915*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sols", "/.", "Vrules"}]], "Input",
 CellChangeTimes->{{3.641040794893097*^9, 3.6410408010344477`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"did", "\[Rule]", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{
        RowBox[{"iq", " ", "Lf", " ", "Lq", " ", "omega"}], "+", 
        RowBox[{"id", " ", "Lf", " ", "R"}], "+", 
        RowBox[{"if", " ", "Lfd", " ", "Rf"}], "+", 
        RowBox[{
         SqrtBox[
          FractionBox["3", "2"]], " ", "Lf", " ", "Va"}], "-", 
        RowBox[{"Lfd", " ", "vf"}]}], 
       RowBox[{
        RowBox[{"Ld", " ", "Lf"}], "-", 
        SuperscriptBox["Lfd", "2"]}]]}]}], ",", 
    RowBox[{"diq", "\[Rule]", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{
        RowBox[{
         RowBox[{"-", "id"}], " ", "Ld", " ", "omega"}], "+", 
        RowBox[{"if", " ", "Lfd", " ", "omega"}], "+", 
        RowBox[{"iq", " ", "R"}]}], "Lq"]}]}], ",", 
    RowBox[{"dif", "\[Rule]", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{
        RowBox[{"iq", " ", "Lfd", " ", "Lq", " ", "omega"}], "+", 
        RowBox[{"id", " ", "Lfd", " ", "R"}], "+", 
        RowBox[{"if", " ", "Ld", " ", "Rf"}], "+", 
        RowBox[{
         SqrtBox[
          FractionBox["3", "2"]], " ", "Lfd", " ", "Va"}], "-", 
        RowBox[{"Ld", " ", "vf"}]}], 
       RowBox[{
        RowBox[{"Ld", " ", "Lf"}], "-", 
        SuperscriptBox["Lfd", "2"]}]]}]}]}], "}"}], "}"}]], "Output",
 CellChangeTimes->{3.641040801956501*^9, 3.6410408887884674`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Simplified \[Omega]-Input Models", "Section",
 CellChangeTimes->{{3.6416458981239605`*^9, 3.6416459319668965`*^9}, {
   3.641801156429264*^9, 3.64180115810236*^9}, 3.641801213851549*^9}],

Cell["\<\
It is useful to have two different types of machine models which will be \
called: \[Omega]-input and P-input. If the mechanical acceleration equation \
is to be solved outside of the machine model the \[Omega]-input model is used \
otherwise the P-input model is used.\
\>", "Text",
 CellChangeTimes->{{3.6514258459189014`*^9, 3.6514259829347386`*^9}, {
  3.6514260231230373`*^9, 3.6514261144632616`*^9}}],

Cell[CellGroupData[{

Cell["Preliminary Calculations", "Subsection",
 CellChangeTimes->{{3.6417347064273643`*^9, 3.6417347159239073`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sols1", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"Solve", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Eap", "\[Equal]", 
         RowBox[{
          RowBox[{"\[Omega]s", " ", "Mf", " ", "Lfd", " ", 
           RowBox[{"Id", "/", "Lf"}]}], " ", "+", 
          RowBox[{"\[Omega]s", " ", "Mf", " ", "Iff"}]}]}], ",", 
        RowBox[{"Ea", "\[Equal]", 
         RowBox[{"Eap", " ", "-", 
          RowBox[{"\[Omega]s", " ", "Mf", " ", "Lfd", " ", 
           RowBox[{"Id", "/", "Lf"}]}]}]}], " ", ",", 
        RowBox[{"Id", "\[Equal]", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"\[Omega]s", " ", "Lq", " ", 
             RowBox[{"(", 
              RowBox[{"Ea", " ", "-", 
               RowBox[{"Va", " ", 
                RowBox[{"Cos", "[", "\[Theta]", "]"}]}]}], ")"}]}], "+", 
            RowBox[{"R", " ", "Va", " ", 
             RowBox[{"Sin", "[", "\[Theta]", "]"}]}]}], ")"}], "/", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"Ld", " ", "Lq", "  ", 
             RowBox[{"\[Omega]s", "^", "2"}]}], " ", "-", " ", 
            RowBox[{"R", "^", "2"}]}], ")"}]}]}], ",", 
        RowBox[{"Iq", "\[Equal]", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "\[Omega]s"}], " ", "Ld", " ", "Va", " ", 
             RowBox[{"Sin", "[", "\[Theta]", "]"}]}], " ", "+", " ", 
            RowBox[{"R", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Va", " ", 
                RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "-", "Ea"}], 
              ")"}]}]}], ")"}], "/", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"Ld", " ", "Lq", " ", 
             RowBox[{"\[Omega]s", "^", "2"}]}], "-", 
            RowBox[{"R", "^", "2"}]}], " ", ")"}]}]}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"Ea", ",", "Iff", ",", "Id", ",", "Iq"}], "}"}]}], "]"}], "//.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Mf", "\[Rule]", 
       RowBox[{
        SqrtBox[
         RowBox[{"2", "/", "3"}]], "Lfd"}]}], ",", 
      RowBox[{"Ld", "\[Rule]", 
       RowBox[{"Xd", "/", "\[Omega]s"}]}], " ", ",", 
      RowBox[{"Lq", "\[Rule]", 
       RowBox[{"Xq", "/", "\[Omega]s"}]}], ",", 
      RowBox[{"\[Omega]s", "\[Rule]", "omegaS"}]}], "}"}]}], "  ", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6416582455691943`*^9, 3.641658340581629*^9}, {
   3.6416583805379143`*^9, 3.6416583828850484`*^9}, {3.6416584709310846`*^9, 
   3.641658506838138*^9}, {3.641658585998666*^9, 3.6416585900698986`*^9}, 
   3.641658628299085*^9, {3.64165868154113*^9, 3.641658690567647*^9}, {
   3.6416646399407673`*^9, 3.6416648652626553`*^9}, {3.641664896663451*^9, 
   3.6416649351736536`*^9}, {3.641664965774404*^9, 3.6416649726457973`*^9}, {
   3.641665028586997*^9, 3.6416650374145017`*^9}, {3.641665173808303*^9, 
   3.641665177227498*^9}, {3.6417157136213694`*^9, 3.641715740987935*^9}, 
   3.6417157829943376`*^9, {3.641720677806017*^9, 3.641720691029773*^9}, {
   3.6417390753252506`*^9, 3.6417390864878893`*^9}, {3.6417399303141537`*^9, 
   3.6417399810460553`*^9}, {3.641741212307479*^9, 3.6417413441770215`*^9}, 
   3.6417414557404027`*^9, {3.6418059689055223`*^9, 3.6418059932409143`*^9}, 
   3.641816612599413*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Ea", "\[Rule]", 
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"-", "3"}], " ", "Eap", " ", "Lf", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["R", "2"], "-", 
          RowBox[{"Xd", " ", "Xq"}]}], ")"}]}], "+", 
       RowBox[{
        SqrtBox["6"], " ", 
        SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Va", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Xq", " ", 
           RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "-", 
          RowBox[{"R", " ", 
           RowBox[{"Sin", "[", "\[Theta]", "]"}]}]}], ")"}]}]}], 
      RowBox[{
       RowBox[{
        RowBox[{"-", "3"}], " ", "Lf", " ", 
        SuperscriptBox["R", "2"]}], "+", 
       RowBox[{
        SqrtBox["6"], " ", 
        SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Xq"}], "+", 
       RowBox[{"3", " ", "Lf", " ", "Xd", " ", "Xq"}]}]]}], ",", 
    RowBox[{"Iff", "\[Rule]", 
     RowBox[{
      FractionBox[
       RowBox[{
        SqrtBox[
         FractionBox["3", "2"]], " ", "Eap"}], 
       RowBox[{"Lfd", " ", "omegaS"}]], "-", 
      FractionBox[
       RowBox[{"3", " ", "Lfd", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Eap", " ", "Xq"}], "-", 
          RowBox[{"Va", " ", "Xq", " ", 
           RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "+", 
          RowBox[{"R", " ", "Va", " ", 
           RowBox[{"Sin", "[", "\[Theta]", "]"}]}]}], ")"}]}], 
       RowBox[{
        RowBox[{
         RowBox[{"-", "3"}], " ", "Lf", " ", 
         SuperscriptBox["R", "2"]}], "+", 
        RowBox[{
         SqrtBox["6"], " ", 
         SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Xq"}], "+", 
        RowBox[{"3", " ", "Lf", " ", "Xd", " ", "Xq"}]}]]}]}], ",", 
    RowBox[{"Id", "\[Rule]", 
     FractionBox[
      RowBox[{"Lf", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Eap", " ", "Xq"}], "-", 
         RowBox[{"Va", " ", "Xq", " ", 
          RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "+", 
         RowBox[{"R", " ", "Va", " ", 
          RowBox[{"Sin", "[", "\[Theta]", "]"}]}]}], ")"}]}], 
      RowBox[{
       RowBox[{
        RowBox[{"-", "Lf"}], " ", 
        SuperscriptBox["R", "2"]}], "+", 
       RowBox[{
        SqrtBox[
         FractionBox["2", "3"]], " ", 
        SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Xq"}], "+", 
       RowBox[{"Lf", " ", "Xd", " ", "Xq"}]}]]}], ",", 
    RowBox[{"Iq", "\[Rule]", 
     FractionBox[
      RowBox[{
       RowBox[{"3", " ", "Eap", " ", "Lf", " ", "R"}], "-", 
       RowBox[{"3", " ", "Lf", " ", "R", " ", "Va", " ", 
        RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "+", 
       RowBox[{"Va", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SqrtBox["6"], " ", 
           SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
          RowBox[{"3", " ", "Lf", " ", "Xd"}]}], ")"}], " ", 
        RowBox[{"Sin", "[", "\[Theta]", "]"}]}]}], 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         SqrtBox["6"]}], " ", 
        SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Xq"}], "+", 
       RowBox[{"3", " ", "Lf", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["R", "2"], "-", 
          RowBox[{"Xd", " ", "Xq"}]}], ")"}]}]}]]}]}], "}"}], "}"}]], "Output",
 CellChangeTimes->{{3.641816646033325*^9, 3.6418166578510013`*^9}, 
   3.642080948410947*^9, 3.6420883704574633`*^9, 3.6427542492072167`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sols2", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{"sols1", "/.", 
    RowBox[{"R", "\[Rule]", "0"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.641665199375765*^9, 3.641665207252216*^9}, {
  3.6416652535928664`*^9, 3.641665262713388*^9}, {3.641730728836268*^9, 
  3.641730731444417*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Ea", "\[Rule]", 
     FractionBox[
      RowBox[{
       RowBox[{"3", " ", "Eap", " ", "Lf", " ", "Xd"}], "+", 
       RowBox[{
        SqrtBox["6"], " ", 
        SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Va", " ", 
        RowBox[{"Cos", "[", "\[Theta]", "]"}]}]}], 
      RowBox[{
       RowBox[{
        SqrtBox["6"], " ", 
        SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
       RowBox[{"3", " ", "Lf", " ", "Xd"}]}]]}], ",", 
    RowBox[{"Iff", "\[Rule]", 
     FractionBox[
      RowBox[{
       RowBox[{"3", " ", 
        SqrtBox["6"], " ", "Eap", " ", "Lf", " ", "Xd"}], "+", 
       RowBox[{"6", " ", 
        SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Va", " ", 
        RowBox[{"Cos", "[", "\[Theta]", "]"}]}]}], 
      RowBox[{
       RowBox[{"2", " ", 
        SqrtBox["6"], " ", 
        SuperscriptBox["Lfd", "3"], " ", 
        SuperscriptBox["omegaS", "2"]}], "+", 
       RowBox[{"6", " ", "Lf", " ", "Lfd", " ", "omegaS", " ", "Xd"}]}]]}], 
    ",", 
    RowBox[{"Id", "\[Rule]", 
     FractionBox[
      RowBox[{"3", " ", "Lf", " ", 
       RowBox[{"(", 
        RowBox[{"Eap", "-", 
         RowBox[{"Va", " ", 
          RowBox[{"Cos", "[", "\[Theta]", "]"}]}]}], ")"}]}], 
      RowBox[{
       RowBox[{
        SqrtBox["6"], " ", 
        SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
       RowBox[{"3", " ", "Lf", " ", "Xd"}]}]]}], ",", 
    RowBox[{"Iq", "\[Rule]", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{"Va", " ", 
        RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "Xq"]}]}]}], "}"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.6416652084062815`*^9, 3.6416652636144395`*^9, 
  3.641715758489936*^9, 3.6417158202164664`*^9, 3.64172071807432*^9, 
  3.6417307335495377`*^9, 3.6417391330395517`*^9, 3.6417394885458856`*^9, 
  3.641740044805702*^9, 3.6417415952053795`*^9, 3.641743682804784*^9, 
  3.641806392131729*^9, 3.641815629871204*^9, 3.641816687468695*^9, 
  3.642080949006981*^9, 3.642088370850486*^9, 3.642754249800251*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"Ea", ",", "Id", ",", "Iq", ",", "Iff"}], "}"}], "=", 
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{"Ea", ",", "Id", ",", "Iq", ",", "Iff"}], "}"}], "/.", 
    RowBox[{"sols2", "[", 
     RowBox[{"[", "1", "]"}], "]"}]}], "/.", 
   RowBox[{"\[Theta]", "\[Rule]", 
    RowBox[{"delta", "-", "theta"}]}]}]}]], "Input",
 CellChangeTimes->{{3.641738924577628*^9, 3.6417390346009216`*^9}, {
  3.641739148871457*^9, 3.641739149178475*^9}, {3.641739369877098*^9, 
  3.6417393991997757`*^9}, {3.6417394490516267`*^9, 3.6417395264740553`*^9}, {
  3.641739573902768*^9, 3.6417396253547106`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"3", " ", "Eap", " ", "Lf", " ", "Xd"}], "+", 
     RowBox[{
      SqrtBox["6"], " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Va", " ", 
      RowBox[{"Cos", "[", 
       RowBox[{"delta", "-", "theta"}], "]"}]}]}], 
    RowBox[{
     RowBox[{
      SqrtBox["6"], " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
     RowBox[{"3", " ", "Lf", " ", "Xd"}]}]], ",", 
   FractionBox[
    RowBox[{"3", " ", "Lf", " ", 
     RowBox[{"(", 
      RowBox[{"Eap", "-", 
       RowBox[{"Va", " ", 
        RowBox[{"Cos", "[", 
         RowBox[{"delta", "-", "theta"}], "]"}]}]}], ")"}]}], 
    RowBox[{
     RowBox[{
      SqrtBox["6"], " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
     RowBox[{"3", " ", "Lf", " ", "Xd"}]}]], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"Va", " ", 
      RowBox[{"Sin", "[", 
       RowBox[{"delta", "-", "theta"}], "]"}]}], "Xq"]}], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"3", " ", 
      SqrtBox["6"], " ", "Eap", " ", "Lf", " ", "Xd"}], "+", 
     RowBox[{"6", " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Va", " ", 
      RowBox[{"Cos", "[", 
       RowBox[{"delta", "-", "theta"}], "]"}]}]}], 
    RowBox[{
     RowBox[{"2", " ", 
      SqrtBox["6"], " ", 
      SuperscriptBox["Lfd", "3"], " ", 
      SuperscriptBox["omegaS", "2"]}], "+", 
     RowBox[{"6", " ", "Lf", " ", "Lfd", " ", "omegaS", " ", "Xd"}]}]]}], 
  "}"}]], "Output",
 CellChangeTimes->{
  3.641739413283581*^9, 3.6417394521958065`*^9, {3.6417394990524864`*^9, 
   3.641739527102091*^9}, 3.6417395782180147`*^9, 3.6417396315940676`*^9, 
   3.6417400815308027`*^9, 3.6417416075590863`*^9, 3.641743682832785*^9, 
   3.6418063921477304`*^9, 3.641815629896206*^9, 3.641816703051586*^9, 
   3.6420809496300163`*^9, 3.642088370866487*^9, 3.6427542498282523`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PE", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"FullSimplify", "[", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"Ea", " ", 
           RowBox[{"Va", "/", 
            RowBox[{"(", 
             RowBox[{"\[Omega]s", " ", "Lq"}], ")"}]}]}], ")"}], " ", 
         RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ")"}], "/.", 
       RowBox[{"sols2", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], "]"}], "/.", 
     RowBox[{"\[Theta]", "\[Rule]", 
      RowBox[{"delta", "-", "theta"}]}]}], "//.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Mf", "\[Rule]", 
       RowBox[{
        SqrtBox[
         RowBox[{"2", "/", "3"}]], "Lfd"}]}], ",", 
      RowBox[{"Ld", "\[Rule]", 
       RowBox[{"Xd", "/", "\[Omega]s"}]}], " ", ",", 
      RowBox[{"Lq", "\[Rule]", 
       RowBox[{"Xq", "/", "\[Omega]s"}]}], ",", 
      RowBox[{"\[Omega]s", "\[Rule]", "omegaS"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6417307606980906`*^9, 3.6417309168620224`*^9}, {
   3.6417396609827485`*^9, 3.6417396761116138`*^9}, {3.641739755930179*^9, 
   3.6417397580022974`*^9}, {3.6417416313404465`*^9, 
   3.6417416324575105`*^9}, {3.641741703990602*^9, 3.641741712038062*^9}, {
   3.641741930564561*^9, 3.641741931171596*^9}, 3.6418061083915005`*^9, 
   3.6418167174004073`*^9, 3.642752115511176*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{"Va", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"3", " ", "Eap", " ", "Lf", " ", "Xd"}], "+", 
     RowBox[{
      SqrtBox["6"], " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Va", " ", 
      RowBox[{"Cos", "[", 
       RowBox[{"delta", "-", "theta"}], "]"}]}]}], ")"}], " ", 
   RowBox[{"Sin", "[", 
    RowBox[{"delta", "-", "theta"}], "]"}]}], 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SqrtBox["6"], " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
     RowBox[{"3", " ", "Lf", " ", "Xd"}]}], ")"}], " ", "Xq"}]]], "Output",
 CellChangeTimes->{
  3.6417309179800863`*^9, 3.6417396776577024`*^9, 3.6417397602394257`*^9, 
   3.6417401358659105`*^9, {3.6417416157135525`*^9, 3.6417416401939526`*^9}, 
   3.6417417136151524`*^9, 3.641741932573676*^9, 3.641743683374816*^9, 
   3.641806392796767*^9, 3.6418156305622435`*^9, 3.6418167220296717`*^9, 
   3.6420809510860996`*^9, 3.6420883716235304`*^9, 3.64275425101332*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QE", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"FullSimplify", "[", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"Ea", " ", 
            RowBox[{"Va", "/", 
             RowBox[{"(", 
              RowBox[{"\[Omega]s", " ", "Lq"}], ")"}]}]}], ")"}], " ", 
          RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "-", 
         RowBox[{
          RowBox[{"Ea", "^", "2"}], "/", 
          RowBox[{"(", 
           RowBox[{"\[Omega]s", " ", "Lq"}], ")"}]}]}], ")"}], "/.", 
       RowBox[{"sols2", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], "]"}], "/.", 
     RowBox[{"\[Theta]", "\[Rule]", 
      RowBox[{"delta", "-", "theta"}]}]}], "//.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Mf", "\[Rule]", 
       RowBox[{
        SqrtBox[
         RowBox[{"2", "/", "3"}]], "Lfd"}]}], ",", 
      RowBox[{"Ld", "\[Rule]", 
       RowBox[{"Xd", "/", "\[Omega]s"}]}], " ", ",", 
      RowBox[{"Lq", "\[Rule]", 
       RowBox[{"Xq", "/", "\[Omega]s"}]}], ",", 
      RowBox[{"\[Omega]s", "\[Rule]", "omegaS"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6416651848689356`*^9, 3.641665190073233*^9}, {
   3.641730968173958*^9, 3.6417310356768184`*^9}, 3.641739696170761*^9, {
   3.6417397770343866`*^9, 3.6417397781834517`*^9}, {3.641741650915566*^9, 
   3.6417416946160655`*^9}, {3.6417419374009523`*^9, 3.641741938083991*^9}, 
   3.6418061231633453`*^9, {3.6418167381235924`*^9, 3.6418167387006254`*^9}, 
   3.641816772778575*^9}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{"3", " ", "Lf", " ", "Xd", " ", 
    RowBox[{"(", 
     RowBox[{"Eap", "-", 
      RowBox[{"Va", " ", 
       RowBox[{"Cos", "[", 
        RowBox[{"delta", "-", "theta"}], "]"}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"3", " ", "Eap", " ", "Lf", " ", "Xd"}], "+", 
      RowBox[{
       SqrtBox["6"], " ", 
       SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Va", " ", 
       RowBox[{"Cos", "[", 
        RowBox[{"delta", "-", "theta"}], "]"}]}]}], ")"}]}], 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SqrtBox["6"], " ", 
        SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
       RowBox[{"3", " ", "Lf", " ", "Xd"}]}], ")"}], "2"], " ", 
    "Xq"}]]}]], "Output",
 CellChangeTimes->{
  3.64173104077711*^9, 3.641739699835971*^9, 3.641739781612648*^9, 
   3.641740138680071*^9, 3.641741657124921*^9, 3.6417416988103056`*^9, 
   3.6417419443383493`*^9, 3.641743686679005*^9, 3.641806396110957*^9, 
   3.6418156339054346`*^9, {3.6418167497532578`*^9, 3.641816778395896*^9}, 
   3.642080955177334*^9, 3.642088375665762*^9, 3.6427542567606487`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sigma", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"FullSimplify", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SqrtBox[
         RowBox[{"3", "/", "2"}]], " ", 
        RowBox[{
         RowBox[{"(", " ", 
          RowBox[{"Mf", "^", "2"}], ")"}], "/", 
         RowBox[{"(", 
          RowBox[{"Lf", " ", "Ld"}], ")"}]}]}], ")"}], "/.", 
      RowBox[{"Mf", "\[Rule]", 
       RowBox[{
        SqrtBox[
         RowBox[{"2", "/", "3"}]], "Lfd"}]}]}], "]"}], "//.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Mf", "\[Rule]", 
       RowBox[{
        SqrtBox[
         RowBox[{"2", "/", "3"}]], "Lfd"}]}], ",", 
      RowBox[{"Ld", "\[Rule]", 
       RowBox[{"Xd", "/", "\[Omega]s"}]}], " ", ",", 
      RowBox[{"Lq", "\[Rule]", 
       RowBox[{"Xq", "/", "\[Omega]s"}]}], ",", 
      RowBox[{"\[Omega]s", "\[Rule]", "omegaS"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6417403609717855`*^9, 3.6417405006497746`*^9}, {
   3.6417417573456535`*^9, 3.641741782159073*^9}, 3.641806137621172*^9, {
   3.641815527450346*^9, 3.6418155371979036`*^9}, {3.6418159855995502`*^9, 
   3.6418159965771785`*^9}, {3.641816043649871*^9, 3.6418160563986*^9}, {
   3.6418164096798067`*^9, 3.6418164186393194`*^9}, {3.6418168337730637`*^9, 
   3.6418168340460787`*^9}, 3.6418168792236633`*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   SqrtBox[
    FractionBox["2", "3"]], " ", 
   SuperscriptBox["Lfd", "2"], " ", "omegaS"}], 
  RowBox[{"Lf", " ", "Xd"}]]], "Output",
 CellChangeTimes->{
  3.641816002757532*^9, {3.641816807054535*^9, 3.641816835932187*^9}, 
   3.6418168833578997`*^9, 3.6420809552543383`*^9, 3.6420883757257648`*^9, 
   3.642754256848654*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PG", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"FullSimplify", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Ea", " ", "Va", " ", 
         RowBox[{
          RowBox[{"Sin", "[", "\[Theta]", "]"}], "/", "Xd"}]}], " ", "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Va", "^", "2"}], "/", "2"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"1", "/", "Xq"}], "-", 
           RowBox[{"1", "/", "Xd"}]}], ")"}], " ", 
         RowBox[{"Sin", "[", 
          RowBox[{"2", " ", "\[Theta]"}], "]"}]}]}], "/.", 
       RowBox[{"sols2", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], "]"}], "/.", 
     RowBox[{"\[Theta]", "\[Rule]", 
      RowBox[{"delta", "-", "theta"}]}]}], "//.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Mf", "\[Rule]", 
       RowBox[{
        SqrtBox[
         RowBox[{"2", "/", "3"}]], "Lfd"}]}], ",", 
      RowBox[{"Ld", "\[Rule]", 
       RowBox[{"Xd", "/", "\[Omega]s"}]}], " ", ",", 
      RowBox[{"Lq", "\[Rule]", 
       RowBox[{"Xq", "/", "\[Omega]s"}]}], ",", 
      RowBox[{"\[Omega]s", "\[Rule]", "omegaS"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.641742395845174*^9, 3.641742425047844*^9}, 
   3.6418061542061205`*^9, 3.641816210218398*^9, 3.6418169004448767`*^9, {
   3.6420883148392825`*^9, 3.642088320700618*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{"Va", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"3", " ", "Eap", " ", "Lf", " ", "Xq"}], "+", 
     RowBox[{"Va", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SqrtBox["6"], " ", 
         SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
        RowBox[{"3", " ", "Lf", " ", 
         RowBox[{"(", 
          RowBox[{"Xd", "-", "Xq"}], ")"}]}]}], ")"}], " ", 
      RowBox[{"Cos", "[", 
       RowBox[{"delta", "-", "theta"}], "]"}]}]}], ")"}], " ", 
   RowBox[{"Sin", "[", 
    RowBox[{"delta", "-", "theta"}], "]"}]}], 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SqrtBox["6"], " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
     RowBox[{"3", " ", "Lf", " ", "Xd"}]}], ")"}], " ", "Xq"}]]], "Output",
 CellChangeTimes->{3.6417424288840637`*^9, 3.6417436896371746`*^9, 
  3.6418063989611197`*^9, 3.64181563697661*^9, 3.6418162166907682`*^9, 
  3.641816909961421*^9, 3.6420809584525213`*^9, 3.6420883771848483`*^9, 
  3.64275425887577*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QG", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"FullSimplify", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Va", "^", "2"}], "/", "2"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"1", "/", "Xq"}], "+", 
           RowBox[{"1", "/", "Xd"}]}], ")"}]}], " ", "-", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"Ea", " ", 
           RowBox[{"Va", "/", "Xd"}]}], ")"}], " ", 
         RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Va", "^", "2"}], "/", "2"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "1"}], "/", "Xq"}], "+", 
           RowBox[{"1", "/", "Xd"}]}], ")"}], " ", 
         RowBox[{"Cos", "[", 
          RowBox[{"2", " ", "\[Theta]"}], "]"}]}]}], "/.", 
       RowBox[{"sols2", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], "]"}], "//.", 
     RowBox[{"\[Theta]", "\[Rule]", 
      RowBox[{"delta", "-", "theta"}]}]}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Mf", "\[Rule]", 
       RowBox[{
        SqrtBox[
         RowBox[{"2", "/", "3"}]], "Lfd"}]}], ",", 
      RowBox[{"Ld", "\[Rule]", 
       RowBox[{"Xd", "/", "\[Omega]s"}]}], " ", ",", 
      RowBox[{"Lq", "\[Rule]", 
       RowBox[{"Xq", "/", "\[Omega]s"}]}], ",", 
      RowBox[{"\[Omega]s", "\[Rule]", "omegaS"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6417424406117344`*^9, 3.641742442579847*^9}, {
   3.641742483650196*^9, 3.641742631754667*^9}, {3.6417426649705667`*^9, 
   3.6417427027327266`*^9}, {3.641742758862937*^9, 3.641742780263161*^9}, {
   3.641806168262925*^9, 3.6418061961965227`*^9}, 3.641816928681492*^9}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", 
   RowBox[{"2", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SqrtBox["6"], " ", 
       SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
      RowBox[{"3", " ", "Lf", " ", "Xd"}]}], ")"}], " ", "Xq"}]], 
  RowBox[{"Va", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "6"}], " ", "Eap", " ", "Lf", " ", "Xq", " ", 
      RowBox[{"Cos", "[", 
       RowBox[{"delta", "-", "theta"}], "]"}]}], "+", 
     RowBox[{"Va", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"3", " ", "Lf", " ", 
         RowBox[{"(", 
          RowBox[{"Xd", "+", "Xq"}], ")"}]}], "+", 
        RowBox[{"3", " ", "Lf", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "Xd"}], "+", "Xq"}], ")"}], " ", 
         RowBox[{"Cos", "[", 
          RowBox[{"2", " ", 
           RowBox[{"(", 
            RowBox[{"delta", "-", "theta"}], ")"}]}], "]"}]}], "+", 
        RowBox[{"2", " ", 
         SqrtBox["6"], " ", 
         SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", 
         SuperscriptBox[
          RowBox[{"Sin", "[", 
           RowBox[{"delta", "-", "theta"}], "]"}], "2"]}]}], ")"}]}]}], 
    ")"}]}]}]], "Output",
 CellChangeTimes->{3.6417427899847174`*^9, 3.641743694704464*^9, 
  3.6418064039894075`*^9, 3.6418156421739078`*^9, 3.6418169360079107`*^9, 
  3.642080963767825*^9, 3.6420883824121475`*^9, 3.6427542655831537`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ddelta", "=", 
   RowBox[{"omega", "-", "omegaS"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"dEap", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"FullSimplify", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "Eap"}], " ", "-", 
        RowBox[{"sigma", " ", "Va", " ", 
         RowBox[{"Cos", "[", 
          RowBox[{"delta", "-", "theta"}], "]"}]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "+", "sigma"}], ")"}], " ", "Efd"}]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "+", "sigma"}], ")"}], 
        RowBox[{"(", 
         RowBox[{"Lf", "/", "Rf"}], ")"}]}], ")"}]}], "]"}], "//.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Mf", "\[Rule]", 
       RowBox[{
        SqrtBox[
         RowBox[{"2", "/", "3"}]], "Lfd"}]}], ",", 
      RowBox[{"Ld", "\[Rule]", 
       RowBox[{"Xd", "/", "\[Omega]s"}]}], " ", ",", 
      RowBox[{"Lq", "\[Rule]", 
       RowBox[{"Xq", "/", "\[Omega]s"}]}], ",", 
      RowBox[{"\[Omega]s", "\[Rule]", "omegaS"}]}], "}"}]}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.6416459710521317`*^9, 3.6416459985057025`*^9}, {
   3.6417347679278817`*^9, 3.6417347757003264`*^9}, 3.6417349365425262`*^9, {
   3.641734992992755*^9, 3.6417350359712133`*^9}, {3.6417350732893476`*^9, 
   3.6417351484906487`*^9}, {3.6417406093729935`*^9, 3.641740686560408*^9}, {
   3.641741815796997*^9, 3.6417418290397544`*^9}, {3.641806182397733*^9, 
   3.6418061909702234`*^9}, 3.64181571930532*^9, {3.6418158402012343`*^9, 
   3.641815854044026*^9}, {3.6418165088394785`*^9, 3.6418165226932707`*^9}, 
   3.641816955902049*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{"Rf", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SqrtBox["6"], " ", "Efd", " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "-", 
     RowBox[{"3", " ", "Eap", " ", "Lf", " ", "Xd"}], "+", 
     RowBox[{"3", " ", "Efd", " ", "Lf", " ", "Xd"}], "-", 
     RowBox[{
      SqrtBox["6"], " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Va", " ", 
      RowBox[{"Cos", "[", 
       RowBox[{"delta", "-", "theta"}], "]"}]}]}], ")"}]}], 
  RowBox[{"Lf", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SqrtBox["6"], " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
     RowBox[{"3", " ", "Lf", " ", "Xd"}]}], ")"}]}]]], "Output",
 CellChangeTimes->{
  3.6417406886815295`*^9, 3.6417418313368855`*^9, 3.641743695890532*^9, 
   3.641806405165475*^9, 3.641815642460924*^9, 3.6418157363482943`*^9, 
   3.641815856720179*^9, {3.641816511129609*^9, 3.6418165234173117`*^9}, 
   3.6418169612093525`*^9, 3.642080964225851*^9, 3.642088382858173*^9, 
   3.642754266028179*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["PE"], "Input",
 CellChangeTimes->{{3.642088275520034*^9, 3.642088276071065*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{"Va", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"3", " ", "Eap", " ", "Lf", " ", "Xd"}], "+", 
     RowBox[{
      SqrtBox["6"], " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Va", " ", 
      RowBox[{"Cos", "[", 
       RowBox[{"delta", "-", "theta"}], "]"}]}]}], ")"}], " ", 
   RowBox[{"Sin", "[", 
    RowBox[{"delta", "-", "theta"}], "]"}]}], 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SqrtBox["6"], " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
     RowBox[{"3", " ", "Lf", " ", "Xd"}]}], ")"}], " ", "Xq"}]]], "Output",
 CellChangeTimes->{3.6420882785202055`*^9, 3.6420883828731737`*^9, 
  3.64275426604818*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["ProPac", "Subsubsection",
 CellChangeTimes->{{3.64181559325811*^9, 3.641815608821*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"<<", "ProPac`"}]], "Input",
 CellChangeTimes->{{3.641743601009105*^9, 3.6417436080855103`*^9}}],

Cell[BoxData["\<\"\\nThis is ProPac - Development Version 5.0\\n\\nProPac has \
been successfully loaded. It contains several packages\\nfor multibody \
dynamics, linear control and nonlinear control.\\nThe package names are:\\n   \
GeoTools\\n   Dynamics\\n   ControlL\\n   ControlN\\n   DifferentialForms\\n  \
 ObserverTools\\n   NDTools\\n   VSCTools\\n   MatlabInterface\\n   \
MEXTools\\nAppropriate packages will be loaded automatically as \
required.\\nTo obtain a list of functions in a package and links to \
function\\ndeclarations, enter:\\n   ?ProPac`PackageName`*\\nHelp is also \
available in the Help Browser under Add-ons.\"\>"], "Print",
 CellChangeTimes->{3.6417436094975905`*^9, 3.6417436959495354`*^9, 
  3.6418064052234783`*^9, 3.641815642495926*^9, 3.6418169821605506`*^9, 
  3.6420809642598534`*^9, 3.6420883828971753`*^9, 3.6427542660751815`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
MEX Function Construction Simplified Model\
\>", "Subsection",
 CellChangeTimes->{{3.641734734482969*^9, 3.641734755413166*^9}, {
  3.641801230323491*^9, 3.6418012388299775`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"stateList", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"delta", ",", "Eap"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"inputs", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"omega", ",", " ", "Efd", ",", " ", "Va", ",", " ", "theta"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"passedParams", "=", 
   RowBox[{"{", 
    RowBox[{
    "\"\<X0\>\"", ",", "\"\<Xd\>\"", ",", "\"\<Xq\>\"", ",", "\"\<Lfd\>\"", 
     ",", "\"\<Lf\>\"", ",", "\"\<Rf\>\"", ",", "\"\<omegaS\>\""}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"passedParamsDimensions", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"2", ",", "1"}], "}"}], "}"}], ",", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{
          RowBox[{"Length", "[", "passedParams", "]"}], "-", "1"}]}], "}"}]}],
       "]"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"outputEquations", "=", 
   RowBox[{"{", 
    RowBox[{
    "delta", ",", "Ea", ",", "Id", ",", "Iq", ",", "Iff", ",", "PE", ",", 
     "QE", ",", "PG", ",", "QG"}], "}"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"stateEquations", "=", 
   RowBox[{"{", 
    RowBox[{"ddelta", ",", "dEap"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
  "mexFilename", "=", 
   "\"\<C:\\\\Users\\\\Harry\\\\Documents\\\\Magic Briefcase\\\\TexProjects\\\
\\PSBook\\\\Notebooks\\\\synchgenerator.c\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CreateControllerMEX", "[", 
   RowBox[{
   "stateList", ",", "inputs", ",", "outputEquations", ",", "stateEquations", 
    ",", "passedParams", ",", "passedParamsDimensions", ",", "mexFilename"}], 
   "]"}], ";"}]}], "Input",
 CellChangeTimes->{{3.6417318753918476`*^9, 3.64173193977353*^9}, {
   3.6417319750935497`*^9, 3.6417320909111743`*^9}, {3.641732138235881*^9, 
   3.641732147365403*^9}, {3.641734906500808*^9, 3.6417349168353987`*^9}, {
   3.6417352637762427`*^9, 3.641735288768672*^9}, {3.641741882605818*^9, 
   3.6417419031599936`*^9}, {3.6417429096305604`*^9, 3.641742963309631*^9}, {
   3.6417430751080256`*^9, 3.6417430809033566`*^9}, {3.6417431400297384`*^9, 
   3.6417431463641005`*^9}, {3.641743405735936*^9, 3.64174340772005*^9}, 
   3.6418108715820456`*^9, {3.6418170000485735`*^9, 3.6418170129663124`*^9}}],

Cell[CellGroupData[{

Cell[BoxData["\<\"  *** MEXTools successfully loaded ***\"\>"], "Print",
 CellChangeTimes->{
  3.641743621625284*^9, 3.641743696152547*^9, 3.64180640543049*^9, {
   3.6418064458117995`*^9, 3.6418064729403515`*^9}, 3.6418170293882523`*^9, 
   3.6420809830969305`*^9, 3.64208838298718*^9}],

Cell[BoxData["\<\"...Generating Header Code\"\>"], "Print",
 CellChangeTimes->{
  3.641743621625284*^9, 3.641743696152547*^9, 3.64180640543049*^9, {
   3.6418064458117995`*^9, 3.6418064729403515`*^9}, 3.6418170293882523`*^9, 
   3.6420809830969305`*^9, 3.6420883829891806`*^9}],

Cell[BoxData["\<\"...Generating Initial Condition Function Code\"\>"], "Print",
 CellChangeTimes->{
  3.641743621625284*^9, 3.641743696152547*^9, 3.64180640543049*^9, {
   3.6418064458117995`*^9, 3.6418064729403515`*^9}, 3.6418170293882523`*^9, 
   3.6420809830969305`*^9, 3.6420883829911804`*^9}],

Cell[BoxData["\<\"...Generating State Derivative Function Code\"\>"], "Print",
 CellChangeTimes->{
  3.641743621625284*^9, 3.641743696152547*^9, 3.64180640543049*^9, {
   3.6418064458117995`*^9, 3.6418064729403515`*^9}, 3.6418170293882523`*^9, 
   3.6420809830969305`*^9, 3.6420883829931803`*^9}],

Cell[BoxData["\<\"...Removing Function terms\"\>"], "Print",
 CellChangeTimes->{
  3.641743621625284*^9, 3.641743696152547*^9, 3.64180640543049*^9, {
   3.6418064458117995`*^9, 3.6418064729403515`*^9}, 3.6418170293882523`*^9, 
   3.6420809830969305`*^9, 3.6420883829961805`*^9}],

Cell[BoxData["\<\"...Generating temp variable declarations\"\>"], "Print",
 CellChangeTimes->{
  3.641743621625284*^9, 3.641743696152547*^9, 3.64180640543049*^9, {
   3.6418064458117995`*^9, 3.6418064729403515`*^9}, 3.6418170293882523`*^9, 
   3.6420809830969305`*^9, 3.6420883830091815`*^9}],

Cell[BoxData["\<\"...Converting Expressions to C form\"\>"], "Print",
 CellChangeTimes->{
  3.641743621625284*^9, 3.641743696152547*^9, 3.64180640543049*^9, {
   3.6418064458117995`*^9, 3.6418064729403515`*^9}, 3.6418170293882523`*^9, 
   3.6420809830969305`*^9, 3.6420883830121813`*^9}],

Cell[BoxData["\<\"...Generating Output Function Code\"\>"], "Print",
 CellChangeTimes->{
  3.641743621625284*^9, 3.641743696152547*^9, 3.64180640543049*^9, {
   3.6418064458117995`*^9, 3.6418064729403515`*^9}, 3.6418170293882523`*^9, 
   3.6420809830969305`*^9, 3.642088383069185*^9}],

Cell[BoxData["\<\"MEX File created with name: \
C:\\\\Users\\\\Harry\\\\Documents\\\\Magic Briefcase\\\\TexProjects\\\\PSBook\
\\\\Notebooks\\\\synchgenerator.c\"\>"], "Print",
 CellChangeTimes->{
  3.641743621625284*^9, 3.641743696152547*^9, 3.64180640543049*^9, {
   3.6418064458117995`*^9, 3.6418064729403515`*^9}, 3.6418170293882523`*^9, 
   3.6420809830969305`*^9, 3.642088383086186*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
MEX Function Construction Reference Model\
\>", "Subsection",
 CellChangeTimes->{{3.641734734482969*^9, 3.641734755413166*^9}, {
  3.641801230323491*^9, 3.6418012649974737`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"stateList", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"delta", ",", "Eap"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"inputs", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
    "omega", ",", " ", "Efd", ",", " ", "Va", ",", " ", "theta", ",", "Pe", 
     ",", "Qe"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"passedParams", "=", 
   RowBox[{"{", 
    RowBox[{
    "\"\<X0\>\"", ",", "\"\<Xd\>\"", ",", "\"\<Xq\>\"", ",", "\"\<Lfd\>\"", 
     ",", "\"\<Lf\>\"", ",", "\"\<Rf\>\"", ",", "\"\<omegaS\>\""}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"passedParamsDimensions", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"2", ",", "1"}], "}"}], "}"}], ",", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{
          RowBox[{"Length", "[", "passedParams", "]"}], "-", "1"}]}], "}"}]}],
       "]"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"outputEquations", "=", 
   RowBox[{"{", 
    RowBox[{
    "delta", ",", "Ea", ",", "Id", ",", "Iq", ",", "Iff", ",", "Pe", ",", 
     "Qe", ",", "PG", ",", "QG"}], "}"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"stateEquations", "=", 
   RowBox[{"{", 
    RowBox[{"ddelta", ",", "dEap"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
  "mexFilename", "=", 
   "\"\<C:\\\\Users\\\\Harry\\\\Documents\\\\Magic Briefcase\\\\TexProjects\\\
\\PSBook\\\\Notebooks\\\\refgenerator.c\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CreateControllerMEX", "[", 
   RowBox[{
   "stateList", ",", "inputs", ",", "outputEquations", ",", "stateEquations", 
    ",", "passedParams", ",", "passedParamsDimensions", ",", "mexFilename"}], 
   "]"}], ";"}]}], "Input",
 CellChangeTimes->{{3.6417318753918476`*^9, 3.64173193977353*^9}, {
   3.6417319750935497`*^9, 3.6417320909111743`*^9}, {3.641732138235881*^9, 
   3.641732147365403*^9}, {3.641734906500808*^9, 3.6417349168353987`*^9}, {
   3.6417352637762427`*^9, 3.641735288768672*^9}, {3.641741882605818*^9, 
   3.6417419031599936`*^9}, {3.6417429096305604`*^9, 3.641742963309631*^9}, {
   3.6417430751080256`*^9, 3.6417430809033566`*^9}, {3.6417431400297384`*^9, 
   3.6417431463641005`*^9}, {3.641743405735936*^9, 3.64174340772005*^9}, {
   3.6418013265659957`*^9, 3.641801351152402*^9}, {3.641801405933535*^9, 
   3.641801448137949*^9}, 3.6418019000337963`*^9, 3.641810847760683*^9, {
   3.641817928125657*^9, 3.641817931047824*^9}}],

Cell[CellGroupData[{

Cell[BoxData["\<\"...Generating Header Code\"\>"], "Print",
 CellChangeTimes->{3.6418180276693506`*^9, 3.6420883944318347`*^9}],

Cell[BoxData["\<\"...Generating Initial Condition Function Code\"\>"], "Print",
 CellChangeTimes->{3.6418180276693506`*^9, 3.642088394433835*^9}],

Cell[BoxData["\<\"...Generating State Derivative Function Code\"\>"], "Print",
 CellChangeTimes->{3.6418180276693506`*^9, 3.642088394435835*^9}],

Cell[BoxData["\<\"...Removing Function terms\"\>"], "Print",
 CellChangeTimes->{3.6418180276693506`*^9, 3.6420883944368353`*^9}],

Cell[BoxData["\<\"...Generating temp variable declarations\"\>"], "Print",
 CellChangeTimes->{3.6418180276693506`*^9, 3.642088394449836*^9}],

Cell[BoxData["\<\"...Converting Expressions to C form\"\>"], "Print",
 CellChangeTimes->{3.6418180276693506`*^9, 3.642088394451836*^9}],

Cell[BoxData["\<\"...Generating Output Function Code\"\>"], "Print",
 CellChangeTimes->{3.6418180276693506`*^9, 3.6420883945178394`*^9}],

Cell[BoxData["\<\"MEX File created with name: \
C:\\\\Users\\\\Harry\\\\Documents\\\\Magic Briefcase\\\\TexProjects\\\\PSBook\
\\\\Notebooks\\\\refgenerator.c\"\>"], "Print",
 CellChangeTimes->{3.6418180276693506`*^9, 3.6420883945248404`*^9}]
}, Open  ]]
}, Closed]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Simplified P-Input Models", "Section",
 CellChangeTimes->{{3.6416458981239605`*^9, 3.6416459319668965`*^9}, {
   3.641801156429264*^9, 3.64180115810236*^9}, 3.641801213851549*^9, {
   3.651425799328237*^9, 3.651425801143341*^9}}],

Cell["\<\
In the P-input case mechanical power is an input to the model and speed is an \
output. Both the kinematic and dynamic mechanical equations are included \
within.the model.\
\>", "Text",
 CellChangeTimes->{{3.6514262879521847`*^9, 3.651426406858986*^9}, {
  3.6514264400748854`*^9, 3.651426448352359*^9}}],

Cell[CellGroupData[{

Cell["Preliminary Calculations", "Subsection",
 CellChangeTimes->{{3.6417347064273643`*^9, 3.6417347159239073`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sols1", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"Solve", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Eap", "\[Equal]", 
         RowBox[{
          RowBox[{"\[Omega]s", " ", "Mf", " ", "Lfd", " ", 
           RowBox[{"Id", "/", "Lf"}]}], " ", "+", 
          RowBox[{"\[Omega]s", " ", "Mf", " ", "Iff"}]}]}], ",", 
        RowBox[{"Ea", "\[Equal]", 
         RowBox[{"Eap", " ", "-", 
          RowBox[{"\[Omega]s", " ", "Mf", " ", "Lfd", " ", 
           RowBox[{"Id", "/", "Lf"}]}]}]}], " ", ",", 
        RowBox[{"Id", "\[Equal]", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{"\[Omega]s", " ", "Lq", " ", 
             RowBox[{"(", 
              RowBox[{"Ea", " ", "-", 
               RowBox[{"Va", " ", 
                RowBox[{"Cos", "[", "\[Theta]", "]"}]}]}], ")"}]}], "+", 
            RowBox[{"R", " ", "Va", " ", 
             RowBox[{"Sin", "[", "\[Theta]", "]"}]}]}], ")"}], "/", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"Ld", " ", "Lq", "  ", 
             RowBox[{"\[Omega]s", "^", "2"}]}], " ", "-", " ", 
            RowBox[{"R", "^", "2"}]}], ")"}]}]}], ",", 
        RowBox[{"Iq", "\[Equal]", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "\[Omega]s"}], " ", "Ld", " ", "Va", " ", 
             RowBox[{"Sin", "[", "\[Theta]", "]"}]}], " ", "+", " ", 
            RowBox[{"R", " ", 
             RowBox[{"(", 
              RowBox[{
               RowBox[{"Va", " ", 
                RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "-", "Ea"}], 
              ")"}]}]}], ")"}], "/", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"Ld", " ", "Lq", " ", 
             RowBox[{"\[Omega]s", "^", "2"}]}], "-", 
            RowBox[{"R", "^", "2"}]}], " ", ")"}]}]}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"Ea", ",", "Iff", ",", "Id", ",", "Iq"}], "}"}]}], "]"}], "//.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Mf", "\[Rule]", 
       RowBox[{
        SqrtBox[
         RowBox[{"2", "/", "3"}]], "Lfd"}]}], ",", 
      RowBox[{"Ld", "\[Rule]", 
       RowBox[{"Xd", "/", "\[Omega]s"}]}], " ", ",", 
      RowBox[{"Lq", "\[Rule]", 
       RowBox[{"Xq", "/", "\[Omega]s"}]}], ",", 
      RowBox[{"\[Omega]s", "\[Rule]", "omegaS"}]}], "}"}]}], "  ", 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.6416582455691943`*^9, 3.641658340581629*^9}, {
   3.6416583805379143`*^9, 3.6416583828850484`*^9}, {3.6416584709310846`*^9, 
   3.641658506838138*^9}, {3.641658585998666*^9, 3.6416585900698986`*^9}, 
   3.641658628299085*^9, {3.64165868154113*^9, 3.641658690567647*^9}, {
   3.6416646399407673`*^9, 3.6416648652626553`*^9}, {3.641664896663451*^9, 
   3.6416649351736536`*^9}, {3.641664965774404*^9, 3.6416649726457973`*^9}, {
   3.641665028586997*^9, 3.6416650374145017`*^9}, {3.641665173808303*^9, 
   3.641665177227498*^9}, {3.6417157136213694`*^9, 3.641715740987935*^9}, 
   3.6417157829943376`*^9, {3.641720677806017*^9, 3.641720691029773*^9}, {
   3.6417390753252506`*^9, 3.6417390864878893`*^9}, {3.6417399303141537`*^9, 
   3.6417399810460553`*^9}, {3.641741212307479*^9, 3.6417413441770215`*^9}, 
   3.6417414557404027`*^9, {3.6418059689055223`*^9, 3.6418059932409143`*^9}, 
   3.641816612599413*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Ea", "\[Rule]", 
     FractionBox[
      RowBox[{
       RowBox[{
        RowBox[{"-", "3"}], " ", "Eap", " ", "Lf", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["R", "2"], "-", 
          RowBox[{"Xd", " ", "Xq"}]}], ")"}]}], "+", 
       RowBox[{
        SqrtBox["6"], " ", 
        SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Va", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Xq", " ", 
           RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "-", 
          RowBox[{"R", " ", 
           RowBox[{"Sin", "[", "\[Theta]", "]"}]}]}], ")"}]}]}], 
      RowBox[{
       RowBox[{
        RowBox[{"-", "3"}], " ", "Lf", " ", 
        SuperscriptBox["R", "2"]}], "+", 
       RowBox[{
        SqrtBox["6"], " ", 
        SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Xq"}], "+", 
       RowBox[{"3", " ", "Lf", " ", "Xd", " ", "Xq"}]}]]}], ",", 
    RowBox[{"Iff", "\[Rule]", 
     RowBox[{
      FractionBox[
       RowBox[{
        SqrtBox[
         FractionBox["3", "2"]], " ", "Eap"}], 
       RowBox[{"Lfd", " ", "omegaS"}]], "-", 
      FractionBox[
       RowBox[{"3", " ", "Lfd", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Eap", " ", "Xq"}], "-", 
          RowBox[{"Va", " ", "Xq", " ", 
           RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "+", 
          RowBox[{"R", " ", "Va", " ", 
           RowBox[{"Sin", "[", "\[Theta]", "]"}]}]}], ")"}]}], 
       RowBox[{
        RowBox[{
         RowBox[{"-", "3"}], " ", "Lf", " ", 
         SuperscriptBox["R", "2"]}], "+", 
        RowBox[{
         SqrtBox["6"], " ", 
         SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Xq"}], "+", 
        RowBox[{"3", " ", "Lf", " ", "Xd", " ", "Xq"}]}]]}]}], ",", 
    RowBox[{"Id", "\[Rule]", 
     FractionBox[
      RowBox[{"Lf", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Eap", " ", "Xq"}], "-", 
         RowBox[{"Va", " ", "Xq", " ", 
          RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "+", 
         RowBox[{"R", " ", "Va", " ", 
          RowBox[{"Sin", "[", "\[Theta]", "]"}]}]}], ")"}]}], 
      RowBox[{
       RowBox[{
        RowBox[{"-", "Lf"}], " ", 
        SuperscriptBox["R", "2"]}], "+", 
       RowBox[{
        SqrtBox[
         FractionBox["2", "3"]], " ", 
        SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Xq"}], "+", 
       RowBox[{"Lf", " ", "Xd", " ", "Xq"}]}]]}], ",", 
    RowBox[{"Iq", "\[Rule]", 
     FractionBox[
      RowBox[{
       RowBox[{"3", " ", "Eap", " ", "Lf", " ", "R"}], "-", 
       RowBox[{"3", " ", "Lf", " ", "R", " ", "Va", " ", 
        RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "+", 
       RowBox[{"Va", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SqrtBox["6"], " ", 
           SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
          RowBox[{"3", " ", "Lf", " ", "Xd"}]}], ")"}], " ", 
        RowBox[{"Sin", "[", "\[Theta]", "]"}]}]}], 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         SqrtBox["6"]}], " ", 
        SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Xq"}], "+", 
       RowBox[{"3", " ", "Lf", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["R", "2"], "-", 
          RowBox[{"Xd", " ", "Xq"}]}], ")"}]}]}]]}]}], "}"}], "}"}]], "Output",
 CellChangeTimes->{{3.641816646033325*^9, 3.6418166578510013`*^9}, 
   3.642080948410947*^9, 3.6420883704574633`*^9, 3.6427542492072167`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sols2", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{"sols1", "/.", 
    RowBox[{"R", "\[Rule]", "0"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.641665199375765*^9, 3.641665207252216*^9}, {
  3.6416652535928664`*^9, 3.641665262713388*^9}, {3.641730728836268*^9, 
  3.641730731444417*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Ea", "\[Rule]", 
     FractionBox[
      RowBox[{
       RowBox[{"3", " ", "Eap", " ", "Lf", " ", "Xd"}], "+", 
       RowBox[{
        SqrtBox["6"], " ", 
        SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Va", " ", 
        RowBox[{"Cos", "[", "\[Theta]", "]"}]}]}], 
      RowBox[{
       RowBox[{
        SqrtBox["6"], " ", 
        SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
       RowBox[{"3", " ", "Lf", " ", "Xd"}]}]]}], ",", 
    RowBox[{"Iff", "\[Rule]", 
     FractionBox[
      RowBox[{
       RowBox[{"3", " ", 
        SqrtBox["6"], " ", "Eap", " ", "Lf", " ", "Xd"}], "+", 
       RowBox[{"6", " ", 
        SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Va", " ", 
        RowBox[{"Cos", "[", "\[Theta]", "]"}]}]}], 
      RowBox[{
       RowBox[{"2", " ", 
        SqrtBox["6"], " ", 
        SuperscriptBox["Lfd", "3"], " ", 
        SuperscriptBox["omegaS", "2"]}], "+", 
       RowBox[{"6", " ", "Lf", " ", "Lfd", " ", "omegaS", " ", "Xd"}]}]]}], 
    ",", 
    RowBox[{"Id", "\[Rule]", 
     FractionBox[
      RowBox[{"3", " ", "Lf", " ", 
       RowBox[{"(", 
        RowBox[{"Eap", "-", 
         RowBox[{"Va", " ", 
          RowBox[{"Cos", "[", "\[Theta]", "]"}]}]}], ")"}]}], 
      RowBox[{
       RowBox[{
        SqrtBox["6"], " ", 
        SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
       RowBox[{"3", " ", "Lf", " ", "Xd"}]}]]}], ",", 
    RowBox[{"Iq", "\[Rule]", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{"Va", " ", 
        RowBox[{"Sin", "[", "\[Theta]", "]"}]}], "Xq"]}]}]}], "}"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.6416652084062815`*^9, 3.6416652636144395`*^9, 
  3.641715758489936*^9, 3.6417158202164664`*^9, 3.64172071807432*^9, 
  3.6417307335495377`*^9, 3.6417391330395517`*^9, 3.6417394885458856`*^9, 
  3.641740044805702*^9, 3.6417415952053795`*^9, 3.641743682804784*^9, 
  3.641806392131729*^9, 3.641815629871204*^9, 3.641816687468695*^9, 
  3.642080949006981*^9, 3.642088370850486*^9, 3.642754249800251*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"Ea", ",", "Id", ",", "Iq", ",", "Iff"}], "}"}], "=", 
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{"Ea", ",", "Id", ",", "Iq", ",", "Iff"}], "}"}], "/.", 
    RowBox[{"sols2", "[", 
     RowBox[{"[", "1", "]"}], "]"}]}], "/.", 
   RowBox[{"\[Theta]", "\[Rule]", 
    RowBox[{"delta", "-", "theta"}]}]}]}]], "Input",
 CellChangeTimes->{{3.641738924577628*^9, 3.6417390346009216`*^9}, {
  3.641739148871457*^9, 3.641739149178475*^9}, {3.641739369877098*^9, 
  3.6417393991997757`*^9}, {3.6417394490516267`*^9, 3.6417395264740553`*^9}, {
  3.641739573902768*^9, 3.6417396253547106`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"3", " ", "Eap", " ", "Lf", " ", "Xd"}], "+", 
     RowBox[{
      SqrtBox["6"], " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Va", " ", 
      RowBox[{"Cos", "[", 
       RowBox[{"delta", "-", "theta"}], "]"}]}]}], 
    RowBox[{
     RowBox[{
      SqrtBox["6"], " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
     RowBox[{"3", " ", "Lf", " ", "Xd"}]}]], ",", 
   FractionBox[
    RowBox[{"3", " ", "Lf", " ", 
     RowBox[{"(", 
      RowBox[{"Eap", "-", 
       RowBox[{"Va", " ", 
        RowBox[{"Cos", "[", 
         RowBox[{"delta", "-", "theta"}], "]"}]}]}], ")"}]}], 
    RowBox[{
     RowBox[{
      SqrtBox["6"], " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
     RowBox[{"3", " ", "Lf", " ", "Xd"}]}]], ",", 
   RowBox[{"-", 
    FractionBox[
     RowBox[{"Va", " ", 
      RowBox[{"Sin", "[", 
       RowBox[{"delta", "-", "theta"}], "]"}]}], "Xq"]}], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"3", " ", 
      SqrtBox["6"], " ", "Eap", " ", "Lf", " ", "Xd"}], "+", 
     RowBox[{"6", " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Va", " ", 
      RowBox[{"Cos", "[", 
       RowBox[{"delta", "-", "theta"}], "]"}]}]}], 
    RowBox[{
     RowBox[{"2", " ", 
      SqrtBox["6"], " ", 
      SuperscriptBox["Lfd", "3"], " ", 
      SuperscriptBox["omegaS", "2"]}], "+", 
     RowBox[{"6", " ", "Lf", " ", "Lfd", " ", "omegaS", " ", "Xd"}]}]]}], 
  "}"}]], "Output",
 CellChangeTimes->{
  3.641739413283581*^9, 3.6417394521958065`*^9, {3.6417394990524864`*^9, 
   3.641739527102091*^9}, 3.6417395782180147`*^9, 3.6417396315940676`*^9, 
   3.6417400815308027`*^9, 3.6417416075590863`*^9, 3.641743682832785*^9, 
   3.6418063921477304`*^9, 3.641815629896206*^9, 3.641816703051586*^9, 
   3.6420809496300163`*^9, 3.642088370866487*^9, 3.6427542498282523`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PE", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"FullSimplify", "[", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"Ea", " ", 
           RowBox[{"Va", "/", 
            RowBox[{"(", 
             RowBox[{"\[Omega]s", " ", "Lq"}], ")"}]}]}], ")"}], " ", 
         RowBox[{"Sin", "[", "\[Theta]", "]"}]}], ")"}], "/.", 
       RowBox[{"sols2", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], "]"}], "/.", 
     RowBox[{"\[Theta]", "\[Rule]", 
      RowBox[{"delta", "-", "theta"}]}]}], "//.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Mf", "\[Rule]", 
       RowBox[{
        SqrtBox[
         RowBox[{"2", "/", "3"}]], "Lfd"}]}], ",", 
      RowBox[{"Ld", "\[Rule]", 
       RowBox[{"Xd", "/", "\[Omega]s"}]}], " ", ",", 
      RowBox[{"Lq", "\[Rule]", 
       RowBox[{"Xq", "/", "\[Omega]s"}]}], ",", 
      RowBox[{"\[Omega]s", "\[Rule]", "omegaS"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6417307606980906`*^9, 3.6417309168620224`*^9}, {
   3.6417396609827485`*^9, 3.6417396761116138`*^9}, {3.641739755930179*^9, 
   3.6417397580022974`*^9}, {3.6417416313404465`*^9, 
   3.6417416324575105`*^9}, {3.641741703990602*^9, 3.641741712038062*^9}, {
   3.641741930564561*^9, 3.641741931171596*^9}, 3.6418061083915005`*^9, 
   3.6418167174004073`*^9, 3.642752115511176*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{"Va", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"3", " ", "Eap", " ", "Lf", " ", "Xd"}], "+", 
     RowBox[{
      SqrtBox["6"], " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Va", " ", 
      RowBox[{"Cos", "[", 
       RowBox[{"delta", "-", "theta"}], "]"}]}]}], ")"}], " ", 
   RowBox[{"Sin", "[", 
    RowBox[{"delta", "-", "theta"}], "]"}]}], 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SqrtBox["6"], " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
     RowBox[{"3", " ", "Lf", " ", "Xd"}]}], ")"}], " ", "Xq"}]]], "Output",
 CellChangeTimes->{
  3.6417309179800863`*^9, 3.6417396776577024`*^9, 3.6417397602394257`*^9, 
   3.6417401358659105`*^9, {3.6417416157135525`*^9, 3.6417416401939526`*^9}, 
   3.6417417136151524`*^9, 3.641741932573676*^9, 3.641743683374816*^9, 
   3.641806392796767*^9, 3.6418156305622435`*^9, 3.6418167220296717`*^9, 
   3.6420809510860996`*^9, 3.6420883716235304`*^9, 3.64275425101332*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QE", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"FullSimplify", "[", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"Ea", " ", 
            RowBox[{"Va", "/", 
             RowBox[{"(", 
              RowBox[{"\[Omega]s", " ", "Lq"}], ")"}]}]}], ")"}], " ", 
          RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "-", 
         RowBox[{
          RowBox[{"Ea", "^", "2"}], "/", 
          RowBox[{"(", 
           RowBox[{"\[Omega]s", " ", "Lq"}], ")"}]}]}], ")"}], "/.", 
       RowBox[{"sols2", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], "]"}], "/.", 
     RowBox[{"\[Theta]", "\[Rule]", 
      RowBox[{"delta", "-", "theta"}]}]}], "//.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Mf", "\[Rule]", 
       RowBox[{
        SqrtBox[
         RowBox[{"2", "/", "3"}]], "Lfd"}]}], ",", 
      RowBox[{"Ld", "\[Rule]", 
       RowBox[{"Xd", "/", "\[Omega]s"}]}], " ", ",", 
      RowBox[{"Lq", "\[Rule]", 
       RowBox[{"Xq", "/", "\[Omega]s"}]}], ",", 
      RowBox[{"\[Omega]s", "\[Rule]", "omegaS"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6416651848689356`*^9, 3.641665190073233*^9}, {
   3.641730968173958*^9, 3.6417310356768184`*^9}, 3.641739696170761*^9, {
   3.6417397770343866`*^9, 3.6417397781834517`*^9}, {3.641741650915566*^9, 
   3.6417416946160655`*^9}, {3.6417419374009523`*^9, 3.641741938083991*^9}, 
   3.6418061231633453`*^9, {3.6418167381235924`*^9, 3.6418167387006254`*^9}, 
   3.641816772778575*^9}],

Cell[BoxData[
 RowBox[{"-", 
  FractionBox[
   RowBox[{"3", " ", "Lf", " ", "Xd", " ", 
    RowBox[{"(", 
     RowBox[{"Eap", "-", 
      RowBox[{"Va", " ", 
       RowBox[{"Cos", "[", 
        RowBox[{"delta", "-", "theta"}], "]"}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"3", " ", "Eap", " ", "Lf", " ", "Xd"}], "+", 
      RowBox[{
       SqrtBox["6"], " ", 
       SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Va", " ", 
       RowBox[{"Cos", "[", 
        RowBox[{"delta", "-", "theta"}], "]"}]}]}], ")"}]}], 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SqrtBox["6"], " ", 
        SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
       RowBox[{"3", " ", "Lf", " ", "Xd"}]}], ")"}], "2"], " ", 
    "Xq"}]]}]], "Output",
 CellChangeTimes->{
  3.64173104077711*^9, 3.641739699835971*^9, 3.641739781612648*^9, 
   3.641740138680071*^9, 3.641741657124921*^9, 3.6417416988103056`*^9, 
   3.6417419443383493`*^9, 3.641743686679005*^9, 3.641806396110957*^9, 
   3.6418156339054346`*^9, {3.6418167497532578`*^9, 3.641816778395896*^9}, 
   3.642080955177334*^9, 3.642088375665762*^9, 3.6427542567606487`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sigma", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"FullSimplify", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SqrtBox[
         RowBox[{"3", "/", "2"}]], " ", 
        RowBox[{
         RowBox[{"(", " ", 
          RowBox[{"Mf", "^", "2"}], ")"}], "/", 
         RowBox[{"(", 
          RowBox[{"Lf", " ", "Ld"}], ")"}]}]}], ")"}], "/.", 
      RowBox[{"Mf", "\[Rule]", 
       RowBox[{
        SqrtBox[
         RowBox[{"2", "/", "3"}]], "Lfd"}]}]}], "]"}], "//.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Mf", "\[Rule]", 
       RowBox[{
        SqrtBox[
         RowBox[{"2", "/", "3"}]], "Lfd"}]}], ",", 
      RowBox[{"Ld", "\[Rule]", 
       RowBox[{"Xd", "/", "\[Omega]s"}]}], " ", ",", 
      RowBox[{"Lq", "\[Rule]", 
       RowBox[{"Xq", "/", "\[Omega]s"}]}], ",", 
      RowBox[{"\[Omega]s", "\[Rule]", "omegaS"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6417403609717855`*^9, 3.6417405006497746`*^9}, {
   3.6417417573456535`*^9, 3.641741782159073*^9}, 3.641806137621172*^9, {
   3.641815527450346*^9, 3.6418155371979036`*^9}, {3.6418159855995502`*^9, 
   3.6418159965771785`*^9}, {3.641816043649871*^9, 3.6418160563986*^9}, {
   3.6418164096798067`*^9, 3.6418164186393194`*^9}, {3.6418168337730637`*^9, 
   3.6418168340460787`*^9}, 3.6418168792236633`*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{
   SqrtBox[
    FractionBox["2", "3"]], " ", 
   SuperscriptBox["Lfd", "2"], " ", "omegaS"}], 
  RowBox[{"Lf", " ", "Xd"}]]], "Output",
 CellChangeTimes->{
  3.641816002757532*^9, {3.641816807054535*^9, 3.641816835932187*^9}, 
   3.6418168833578997`*^9, 3.6420809552543383`*^9, 3.6420883757257648`*^9, 
   3.642754256848654*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PG", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"FullSimplify", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Ea", " ", "Va", " ", 
         RowBox[{
          RowBox[{"Sin", "[", "\[Theta]", "]"}], "/", "Xd"}]}], " ", "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Va", "^", "2"}], "/", "2"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"1", "/", "Xq"}], "-", 
           RowBox[{"1", "/", "Xd"}]}], ")"}], " ", 
         RowBox[{"Sin", "[", 
          RowBox[{"2", " ", "\[Theta]"}], "]"}]}]}], "/.", 
       RowBox[{"sols2", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], "]"}], "/.", 
     RowBox[{"\[Theta]", "\[Rule]", 
      RowBox[{"delta", "-", "theta"}]}]}], "//.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Mf", "\[Rule]", 
       RowBox[{
        SqrtBox[
         RowBox[{"2", "/", "3"}]], "Lfd"}]}], ",", 
      RowBox[{"Ld", "\[Rule]", 
       RowBox[{"Xd", "/", "\[Omega]s"}]}], " ", ",", 
      RowBox[{"Lq", "\[Rule]", 
       RowBox[{"Xq", "/", "\[Omega]s"}]}], ",", 
      RowBox[{"\[Omega]s", "\[Rule]", "omegaS"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.641742395845174*^9, 3.641742425047844*^9}, 
   3.6418061542061205`*^9, 3.641816210218398*^9, 3.6418169004448767`*^9, {
   3.6420883148392825`*^9, 3.642088320700618*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{"Va", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"3", " ", "Eap", " ", "Lf", " ", "Xq"}], "+", 
     RowBox[{"Va", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SqrtBox["6"], " ", 
         SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
        RowBox[{"3", " ", "Lf", " ", 
         RowBox[{"(", 
          RowBox[{"Xd", "-", "Xq"}], ")"}]}]}], ")"}], " ", 
      RowBox[{"Cos", "[", 
       RowBox[{"delta", "-", "theta"}], "]"}]}]}], ")"}], " ", 
   RowBox[{"Sin", "[", 
    RowBox[{"delta", "-", "theta"}], "]"}]}], 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SqrtBox["6"], " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
     RowBox[{"3", " ", "Lf", " ", "Xd"}]}], ")"}], " ", "Xq"}]]], "Output",
 CellChangeTimes->{3.6417424288840637`*^9, 3.6417436896371746`*^9, 
  3.6418063989611197`*^9, 3.64181563697661*^9, 3.6418162166907682`*^9, 
  3.641816909961421*^9, 3.6420809584525213`*^9, 3.6420883771848483`*^9, 
  3.64275425887577*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QG", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"FullSimplify", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Va", "^", "2"}], "/", "2"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"1", "/", "Xq"}], "+", 
           RowBox[{"1", "/", "Xd"}]}], ")"}]}], " ", "-", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"Ea", " ", 
           RowBox[{"Va", "/", "Xd"}]}], ")"}], " ", 
         RowBox[{"Cos", "[", "\[Theta]", "]"}]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"Va", "^", "2"}], "/", "2"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "1"}], "/", "Xq"}], "+", 
           RowBox[{"1", "/", "Xd"}]}], ")"}], " ", 
         RowBox[{"Cos", "[", 
          RowBox[{"2", " ", "\[Theta]"}], "]"}]}]}], "/.", 
       RowBox[{"sols2", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], "]"}], "//.", 
     RowBox[{"\[Theta]", "\[Rule]", 
      RowBox[{"delta", "-", "theta"}]}]}], "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Mf", "\[Rule]", 
       RowBox[{
        SqrtBox[
         RowBox[{"2", "/", "3"}]], "Lfd"}]}], ",", 
      RowBox[{"Ld", "\[Rule]", 
       RowBox[{"Xd", "/", "\[Omega]s"}]}], " ", ",", 
      RowBox[{"Lq", "\[Rule]", 
       RowBox[{"Xq", "/", "\[Omega]s"}]}], ",", 
      RowBox[{"\[Omega]s", "\[Rule]", "omegaS"}]}], "}"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.6417424406117344`*^9, 3.641742442579847*^9}, {
   3.641742483650196*^9, 3.641742631754667*^9}, {3.6417426649705667`*^9, 
   3.6417427027327266`*^9}, {3.641742758862937*^9, 3.641742780263161*^9}, {
   3.641806168262925*^9, 3.6418061961965227`*^9}, 3.641816928681492*^9}],

Cell[BoxData[
 RowBox[{
  FractionBox["1", 
   RowBox[{"2", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SqrtBox["6"], " ", 
       SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
      RowBox[{"3", " ", "Lf", " ", "Xd"}]}], ")"}], " ", "Xq"}]], 
  RowBox[{"Va", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "6"}], " ", "Eap", " ", "Lf", " ", "Xq", " ", 
      RowBox[{"Cos", "[", 
       RowBox[{"delta", "-", "theta"}], "]"}]}], "+", 
     RowBox[{"Va", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"3", " ", "Lf", " ", 
         RowBox[{"(", 
          RowBox[{"Xd", "+", "Xq"}], ")"}]}], "+", 
        RowBox[{"3", " ", "Lf", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "Xd"}], "+", "Xq"}], ")"}], " ", 
         RowBox[{"Cos", "[", 
          RowBox[{"2", " ", 
           RowBox[{"(", 
            RowBox[{"delta", "-", "theta"}], ")"}]}], "]"}]}], "+", 
        RowBox[{"2", " ", 
         SqrtBox["6"], " ", 
         SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", 
         SuperscriptBox[
          RowBox[{"Sin", "[", 
           RowBox[{"delta", "-", "theta"}], "]"}], "2"]}]}], ")"}]}]}], 
    ")"}]}]}]], "Output",
 CellChangeTimes->{3.6417427899847174`*^9, 3.641743694704464*^9, 
  3.6418064039894075`*^9, 3.6418156421739078`*^9, 3.6418169360079107`*^9, 
  3.642080963767825*^9, 3.6420883824121475`*^9, 3.6427542655831537`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"ddelta", "=", 
   RowBox[{"omega", "-", "omegaS"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"domega", "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"Pm", "-", "PG", "-", 
      RowBox[{"D", " ", "omega"}]}], ")"}], "/", "M"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"dEap", "=", 
  RowBox[{"FullSimplify", "[", 
   RowBox[{
    RowBox[{"FullSimplify", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "Eap"}], " ", "-", 
        RowBox[{"sigma", " ", "Va", " ", 
         RowBox[{"Cos", "[", 
          RowBox[{"delta", "-", "theta"}], "]"}]}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "+", "sigma"}], ")"}], " ", "Efd"}]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "+", "sigma"}], ")"}], 
        RowBox[{"(", 
         RowBox[{"Lf", "/", "Rf"}], ")"}]}], ")"}]}], "]"}], "//.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Mf", "\[Rule]", 
       RowBox[{
        SqrtBox[
         RowBox[{"2", "/", "3"}]], "Lfd"}]}], ",", 
      RowBox[{"Ld", "\[Rule]", 
       RowBox[{"Xd", "/", "\[Omega]s"}]}], " ", ",", 
      RowBox[{"Lq", "\[Rule]", 
       RowBox[{"Xq", "/", "\[Omega]s"}]}], ",", 
      RowBox[{"\[Omega]s", "\[Rule]", "omegaS"}]}], "}"}]}], 
   "]"}]}]}], "Input",
 CellChangeTimes->{{3.6416459710521317`*^9, 3.6416459985057025`*^9}, {
   3.6417347679278817`*^9, 3.6417347757003264`*^9}, 3.6417349365425262`*^9, {
   3.641734992992755*^9, 3.6417350359712133`*^9}, {3.6417350732893476`*^9, 
   3.6417351484906487`*^9}, {3.6417406093729935`*^9, 3.641740686560408*^9}, {
   3.641741815796997*^9, 3.6417418290397544`*^9}, {3.641806182397733*^9, 
   3.6418061909702234`*^9}, 3.64181571930532*^9, {3.6418158402012343`*^9, 
   3.641815854044026*^9}, {3.6418165088394785`*^9, 3.6418165226932707`*^9}, 
   3.641816955902049*^9, {3.6514265062136683`*^9, 3.6514265117299843`*^9}, {
   3.651426744343289*^9, 3.651426799403438*^9}, 3.6514268322633176`*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{"Rf", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SqrtBox["6"], " ", "Efd", " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "-", 
     RowBox[{"3", " ", "Eap", " ", "Lf", " ", "Xd"}], "+", 
     RowBox[{"3", " ", "Efd", " ", "Lf", " ", "Xd"}], "-", 
     RowBox[{
      SqrtBox["6"], " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Va", " ", 
      RowBox[{"Cos", "[", 
       RowBox[{"delta", "-", "theta"}], "]"}]}]}], ")"}]}], 
  RowBox[{"Lf", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SqrtBox["6"], " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
     RowBox[{"3", " ", "Lf", " ", "Xd"}]}], ")"}]}]]], "Output",
 CellChangeTimes->{
  3.6417406886815295`*^9, 3.6417418313368855`*^9, 3.641743695890532*^9, 
   3.641806405165475*^9, 3.641815642460924*^9, 3.6418157363482943`*^9, 
   3.641815856720179*^9, {3.641816511129609*^9, 3.6418165234173117`*^9}, 
   3.6418169612093525`*^9, 3.642080964225851*^9, 3.642088382858173*^9, 
   3.642754266028179*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["PE"], "Input",
 CellChangeTimes->{{3.642088275520034*^9, 3.642088276071065*^9}}],

Cell[BoxData[
 FractionBox[
  RowBox[{"Va", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"3", " ", "Eap", " ", "Lf", " ", "Xd"}], "+", 
     RowBox[{
      SqrtBox["6"], " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS", " ", "Va", " ", 
      RowBox[{"Cos", "[", 
       RowBox[{"delta", "-", "theta"}], "]"}]}]}], ")"}], " ", 
   RowBox[{"Sin", "[", 
    RowBox[{"delta", "-", "theta"}], "]"}]}], 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SqrtBox["6"], " ", 
      SuperscriptBox["Lfd", "2"], " ", "omegaS"}], "+", 
     RowBox[{"3", " ", "Lf", " ", "Xd"}]}], ")"}], " ", "Xq"}]]], "Output",
 CellChangeTimes->{3.6420882785202055`*^9, 3.6420883828731737`*^9, 
  3.64275426604818*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["ProPac", "Subsubsection",
 CellChangeTimes->{{3.64181559325811*^9, 3.641815608821*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"<<", "ProPac`"}]], "Input",
 CellChangeTimes->{{3.641743601009105*^9, 3.6417436080855103`*^9}}],

Cell[BoxData["\<\"\\nThis is ProPac - Development Version 5.0\\n\\nProPac has \
been successfully loaded. It contains several packages\\nfor multibody \
dynamics, linear control and nonlinear control.\\nThe package names are:\\n   \
GeoTools\\n   Dynamics\\n   ControlL\\n   ControlN\\n   DifferentialForms\\n  \
 ObserverTools\\n   NDTools\\n   VSCTools\\n   MatlabInterface\\n   \
MEXTools\\nAppropriate packages will be loaded automatically as \
required.\\nTo obtain a list of functions in a package and links to \
function\\ndeclarations, enter:\\n   ?ProPac`PackageName`*\\nHelp is also \
available in the Help Browser under Add-ons.\"\>"], "Print",
 CellChangeTimes->{3.6417436094975905`*^9, 3.6417436959495354`*^9, 
  3.6418064052234783`*^9, 3.641815642495926*^9, 3.6418169821605506`*^9, 
  3.6420809642598534`*^9, 3.6420883828971753`*^9, 3.6427542660751815`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MEX Function Construction Simplified Model", "Subsection",
 CellChangeTimes->{{3.641734734482969*^9, 3.641734755413166*^9}, {
  3.641801230323491*^9, 3.6418012388299775`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"stateList", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"omega", ",", "delta", ",", "Eap"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"inputs", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"Pm", ",", " ", "Efd", ",", " ", "Va", ",", " ", "theta"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"passedParams", "=", 
   RowBox[{"{", 
    RowBox[{
    "\"\<X0\>\"", ",", "\"\<Xd\>\"", ",", "\"\<Xq\>\"", ",", "\"\<Lfd\>\"", 
     ",", "\"\<Lf\>\"", ",", "\"\<Rf\>\"", ",", "\"\<omegaS\>\"", ",", 
     "\"\<M\>\"", ",", "\"\<D\>\""}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"passedParamsDimensions", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"3", ",", "1"}], "}"}], "}"}], ",", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{
          RowBox[{"Length", "[", "passedParams", "]"}], "-", "1"}]}], "}"}]}],
       "]"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"outputEquations", "=", 
   RowBox[{"{", 
    RowBox[{
    "omega", ",", "delta", ",", "Ea", ",", "Id", ",", "Iq", ",", "Iff", ",", 
     "PE", ",", "QE", ",", "PG", ",", "QG"}], "}"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"stateEquations", "=", 
   RowBox[{"{", 
    RowBox[{"domega", ",", "ddelta", ",", "dEap"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
  "mexFilename", "=", 
   "\"\<C:\\\\Users\\\\Harry\\\\Documents\\\\Magic Briefcase\\\\TexProjects\\\
\\PSBook\\\\Notebooks\\\\synchgeneratorP.c\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CreateControllerMEX", "[", 
   RowBox[{
   "stateList", ",", "inputs", ",", "outputEquations", ",", "stateEquations", 
    ",", "passedParams", ",", "passedParamsDimensions", ",", "mexFilename"}], 
   "]"}], ";"}]}], "Input",
 CellChangeTimes->{{3.6417318753918476`*^9, 3.64173193977353*^9}, {
   3.6417319750935497`*^9, 3.6417320909111743`*^9}, {3.641732138235881*^9, 
   3.641732147365403*^9}, {3.641734906500808*^9, 3.6417349168353987`*^9}, {
   3.6417352637762427`*^9, 3.641735288768672*^9}, {3.641741882605818*^9, 
   3.6417419031599936`*^9}, {3.6417429096305604`*^9, 3.641742963309631*^9}, {
   3.6417430751080256`*^9, 3.6417430809033566`*^9}, {3.6417431400297384`*^9, 
   3.6417431463641005`*^9}, {3.641743405735936*^9, 3.64174340772005*^9}, 
   3.6418108715820456`*^9, {3.6418170000485735`*^9, 3.6418170129663124`*^9}, {
   3.651426859822894*^9, 3.651426862299035*^9}, {3.6514269038804135`*^9, 
   3.6514269275217657`*^9}, {3.6514269847570395`*^9, 3.651427021813159*^9}, 
   3.6514270585282593`*^9}],

Cell[CellGroupData[{

Cell[BoxData["\<\"  *** MEXTools successfully loaded ***\"\>"], "Print",
 CellChangeTimes->{
  3.641743621625284*^9, 3.641743696152547*^9, 3.64180640543049*^9, {
   3.6418064458117995`*^9, 3.6418064729403515`*^9}, 3.6418170293882523`*^9, 
   3.6420809830969305`*^9, 3.64208838298718*^9}],

Cell[BoxData["\<\"...Generating Header Code\"\>"], "Print",
 CellChangeTimes->{
  3.641743621625284*^9, 3.641743696152547*^9, 3.64180640543049*^9, {
   3.6418064458117995`*^9, 3.6418064729403515`*^9}, 3.6418170293882523`*^9, 
   3.6420809830969305`*^9, 3.6420883829891806`*^9}],

Cell[BoxData["\<\"...Generating Initial Condition Function Code\"\>"], "Print",
 CellChangeTimes->{
  3.641743621625284*^9, 3.641743696152547*^9, 3.64180640543049*^9, {
   3.6418064458117995`*^9, 3.6418064729403515`*^9}, 3.6418170293882523`*^9, 
   3.6420809830969305`*^9, 3.6420883829911804`*^9}],

Cell[BoxData["\<\"...Generating State Derivative Function Code\"\>"], "Print",
 CellChangeTimes->{
  3.641743621625284*^9, 3.641743696152547*^9, 3.64180640543049*^9, {
   3.6418064458117995`*^9, 3.6418064729403515`*^9}, 3.6418170293882523`*^9, 
   3.6420809830969305`*^9, 3.6420883829931803`*^9}],

Cell[BoxData["\<\"...Removing Function terms\"\>"], "Print",
 CellChangeTimes->{
  3.641743621625284*^9, 3.641743696152547*^9, 3.64180640543049*^9, {
   3.6418064458117995`*^9, 3.6418064729403515`*^9}, 3.6418170293882523`*^9, 
   3.6420809830969305`*^9, 3.6420883829961805`*^9}],

Cell[BoxData["\<\"...Generating temp variable declarations\"\>"], "Print",
 CellChangeTimes->{
  3.641743621625284*^9, 3.641743696152547*^9, 3.64180640543049*^9, {
   3.6418064458117995`*^9, 3.6418064729403515`*^9}, 3.6418170293882523`*^9, 
   3.6420809830969305`*^9, 3.6420883830091815`*^9}],

Cell[BoxData["\<\"...Converting Expressions to C form\"\>"], "Print",
 CellChangeTimes->{
  3.641743621625284*^9, 3.641743696152547*^9, 3.64180640543049*^9, {
   3.6418064458117995`*^9, 3.6418064729403515`*^9}, 3.6418170293882523`*^9, 
   3.6420809830969305`*^9, 3.6420883830121813`*^9}],

Cell[BoxData["\<\"...Generating Output Function Code\"\>"], "Print",
 CellChangeTimes->{
  3.641743621625284*^9, 3.641743696152547*^9, 3.64180640543049*^9, {
   3.6418064458117995`*^9, 3.6418064729403515`*^9}, 3.6418170293882523`*^9, 
   3.6420809830969305`*^9, 3.642088383069185*^9}],

Cell[BoxData["\<\"MEX File created with name: \
C:\\\\Users\\\\Harry\\\\Documents\\\\Magic Briefcase\\\\TexProjects\\\\PSBook\
\\\\Notebooks\\\\synchgenerator.c\"\>"], "Print",
 CellChangeTimes->{
  3.641743621625284*^9, 3.641743696152547*^9, 3.64180640543049*^9, {
   3.6418064458117995`*^9, 3.6418064729403515`*^9}, 3.6418170293882523`*^9, 
   3.6420809830969305`*^9, 3.642088383086186*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MEX Function Construction Reference Model", "Subsection",
 CellChangeTimes->{{3.641734734482969*^9, 3.641734755413166*^9}, {
  3.641801230323491*^9, 3.6418012649974737`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"stateList", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"omega", ",", "delta", ",", "Eap"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"inputs", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{
    "omega", ",", " ", "Efd", ",", " ", "Va", ",", " ", "theta", ",", "Pe", 
     ",", "Qe"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"passedParams", "=", 
   RowBox[{"{", 
    RowBox[{
    "\"\<X0\>\"", ",", "\"\<Xd\>\"", ",", "\"\<Xq\>\"", ",", "\"\<Lfd\>\"", 
     ",", "\"\<Lf\>\"", ",", "\"\<Rf\>\"", ",", "\"\<omegaS\>\"", ",", 
     "\"\<M\>\"", ",", "\"\<D\>\""}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"passedParamsDimensions", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"3", ",", "1"}], "}"}], "}"}], ",", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{
          RowBox[{"Length", "[", "passedParams", "]"}], "-", "1"}]}], "}"}]}],
       "]"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"outputEquations", "=", 
   RowBox[{"{", 
    RowBox[{
    "omega", ",", "delta", ",", "Ea", ",", "Id", ",", "Iq", ",", "Iff", ",", 
     "Pe", ",", "Qe", ",", "PG", ",", "QG"}], "}"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"stateEquations", "=", 
   RowBox[{"{", 
    RowBox[{"domega", ",", "ddelta", ",", "dEap"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
  "mexFilename", "=", 
   "\"\<C:\\\\Users\\\\Harry\\\\Documents\\\\Magic Briefcase\\\\TexProjects\\\
\\PSBook\\\\Notebooks\\\\refgeneratorP.c\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CreateControllerMEX", "[", 
   RowBox[{
   "stateList", ",", "inputs", ",", "outputEquations", ",", "stateEquations", 
    ",", "passedParams", ",", "passedParamsDimensions", ",", "mexFilename"}], 
   "]"}], ";"}]}], "Input",
 CellChangeTimes->{{3.6417318753918476`*^9, 3.64173193977353*^9}, {
   3.6417319750935497`*^9, 3.6417320909111743`*^9}, {3.641732138235881*^9, 
   3.641732147365403*^9}, {3.641734906500808*^9, 3.6417349168353987`*^9}, {
   3.6417352637762427`*^9, 3.641735288768672*^9}, {3.641741882605818*^9, 
   3.6417419031599936`*^9}, {3.6417429096305604`*^9, 3.641742963309631*^9}, {
   3.6417430751080256`*^9, 3.6417430809033566`*^9}, {3.6417431400297384`*^9, 
   3.6417431463641005`*^9}, {3.641743405735936*^9, 3.64174340772005*^9}, {
   3.6418013265659957`*^9, 3.641801351152402*^9}, {3.641801405933535*^9, 
   3.641801448137949*^9}, 3.6418019000337963`*^9, 3.641810847760683*^9, {
   3.641817928125657*^9, 3.641817931047824*^9}, {3.6514272649690666`*^9, 
   3.651427327045617*^9}, {3.6514273609505568`*^9, 3.651427371554163*^9}}],

Cell[CellGroupData[{

Cell[BoxData["\<\"...Generating Header Code\"\>"], "Print",
 CellChangeTimes->{3.6418180276693506`*^9, 3.6420883944318347`*^9}],

Cell[BoxData["\<\"...Generating Initial Condition Function Code\"\>"], "Print",
 CellChangeTimes->{3.6418180276693506`*^9, 3.642088394433835*^9}],

Cell[BoxData["\<\"...Generating State Derivative Function Code\"\>"], "Print",
 CellChangeTimes->{3.6418180276693506`*^9, 3.642088394435835*^9}],

Cell[BoxData["\<\"...Removing Function terms\"\>"], "Print",
 CellChangeTimes->{3.6418180276693506`*^9, 3.6420883944368353`*^9}],

Cell[BoxData["\<\"...Generating temp variable declarations\"\>"], "Print",
 CellChangeTimes->{3.6418180276693506`*^9, 3.642088394449836*^9}],

Cell[BoxData["\<\"...Converting Expressions to C form\"\>"], "Print",
 CellChangeTimes->{3.6418180276693506`*^9, 3.642088394451836*^9}],

Cell[BoxData["\<\"...Generating Output Function Code\"\>"], "Print",
 CellChangeTimes->{3.6418180276693506`*^9, 3.6420883945178394`*^9}],

Cell[BoxData["\<\"MEX File created with name: \
C:\\\\Users\\\\Harry\\\\Documents\\\\Magic Briefcase\\\\TexProjects\\\\PSBook\
\\\\Notebooks\\\\refgenerator.c\"\>"], "Print",
 CellChangeTimes->{3.6418180276693506`*^9, 3.6420883945248404`*^9}]
}, Open  ]]
}, Closed]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowSize->{1073, 560},
WindowMargins->{{72, Automatic}, {Automatic, 10}},
PrivateNotebookOptions->{"VersionedStylesheet"->{"Default.nb"[8.] -> False}},
FrontEndVersion->"9.0 for Microsoft Windows (64-bit) (January 25, 2013)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 113, 1, 101, "Title"],
Cell[695, 25, 103, 1, 56, "Subtitle"],
Cell[801, 28, 108, 1, 33, "Subsubtitle"],
Cell[CellGroupData[{
Cell[934, 33, 100, 1, 86, "Section"],
Cell[CellGroupData[{
Cell[1059, 38, 117, 2, 31, "Input"],
Cell[1179, 42, 819, 11, 404, "Print"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2047, 59, 108, 1, 86, "Section"],
Cell[CellGroupData[{
Cell[2180, 64, 1230, 39, 79, "Input"],
Cell[3413, 105, 1987, 64, 134, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5437, 174, 152, 3, 31, "Input"],
Cell[5592, 179, 1386, 50, 59, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[7015, 234, 128, 2, 31, "Input"],
Cell[7146, 238, 1808, 60, 147, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[9003, 304, 156, 2, 86, "Section"],
Cell[9162, 308, 7340, 276, 695, "Text"],
Cell[CellGroupData[{
Cell[16527, 588, 4070, 120, 312, "Input"],
Cell[CellGroupData[{
Cell[20622, 712, 207, 2, 23, "Print"],
Cell[20832, 716, 192, 2, 23, "Print"],
Cell[21027, 720, 212, 2, 23, "Print"],
Cell[21242, 724, 213, 2, 23, "Print"],
Cell[21458, 728, 195, 2, 23, "Print"],
Cell[21656, 732, 209, 2, 23, "Print"],
Cell[21868, 736, 202, 2, 23, "Print"],
Cell[22073, 740, 203, 2, 23, "Print"],
Cell[22279, 744, 380, 6, 43, "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[22720, 757, 207, 3, 86, "Section"],
Cell[CellGroupData[{
Cell[22952, 764, 2982, 77, 232, "Input"],
Cell[CellGroupData[{
Cell[25959, 845, 193, 3, 23, "Print"],
Cell[26155, 850, 180, 3, 23, "Print"],
Cell[26338, 855, 198, 3, 23, "Print"],
Cell[26539, 860, 197, 3, 23, "Print"],
Cell[26739, 865, 179, 3, 23, "Print"],
Cell[26921, 870, 193, 3, 23, "Print"],
Cell[27117, 875, 188, 3, 23, "Print"],
Cell[27308, 880, 189, 3, 23, "Print"],
Cell[27500, 885, 364, 7, 43, "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[27925, 899, 100, 1, 86, "Section"],
Cell[28028, 902, 1752, 47, 82, "Input"],
Cell[CellGroupData[{
Cell[29805, 953, 443, 10, 31, "Input"],
Cell[30251, 965, 1378, 44, 56, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[31666, 1014, 292, 6, 31, "Input"],
Cell[31961, 1022, 314, 7, 36, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[32312, 1034, 244, 6, 31, "Input"],
Cell[32559, 1042, 442, 14, 36, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[33038, 1061, 280, 7, 31, "Input"],
Cell[33321, 1070, 168, 4, 34, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[33526, 1079, 331, 8, 31, "Input"],
Cell[33860, 1089, 460, 12, 34, "Output"]
}, Open  ]],
Cell[34335, 1104, 93, 1, 31, "Input"],
Cell[34431, 1107, 1130, 32, 52, "Input"],
Cell[CellGroupData[{
Cell[35586, 1143, 251, 6, 31, "Input"],
Cell[35840, 1151, 1311, 36, 86, "Output"]
}, Open  ]],
Cell[37166, 1190, 94, 1, 31, "Input"],
Cell[CellGroupData[{
Cell[37285, 1195, 580, 15, 31, "Input"],
Cell[37868, 1212, 306, 9, 56, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[38211, 1226, 126, 2, 31, "Input"],
Cell[38340, 1230, 1416, 40, 149, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[39805, 1276, 192, 2, 86, "Section"],
Cell[40000, 1280, 416, 7, 49, "Text"],
Cell[CellGroupData[{
Cell[40441, 1291, 116, 1, 49, "Subsection"],
Cell[CellGroupData[{
Cell[40582, 1296, 3346, 77, 147, "Input"],
Cell[43931, 1375, 3489, 98, 219, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[47457, 1478, 318, 7, 31, "Input"],
Cell[47778, 1487, 2069, 55, 106, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[49884, 1547, 645, 15, 31, "Input"],
Cell[50532, 1564, 1916, 53, 106, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[52485, 1622, 1394, 35, 67, "Input"],
Cell[53882, 1659, 1013, 25, 61, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[54932, 1689, 1564, 40, 67, "Input"],
Cell[56499, 1731, 1182, 31, 62, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[57718, 1767, 1352, 35, 67, "Input"],
Cell[59073, 1804, 367, 10, 75, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[59477, 1819, 1399, 38, 67, "Input"],
Cell[60879, 1859, 1041, 29, 61, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[61957, 1893, 1839, 50, 107, "Input"],
Cell[63799, 1945, 1426, 40, 80, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[65262, 1990, 1708, 43, 87, "Input"],
Cell[66973, 2035, 1053, 27, 61, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[68063, 2067, 94, 1, 31, "Input"],
Cell[68160, 2070, 713, 21, 61, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[68910, 2096, 93, 1, 39, "Subsubsection"],
Cell[CellGroupData[{
Cell[69028, 2101, 119, 2, 31, "Input"],
Cell[69150, 2105, 871, 11, 404, "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[70082, 2123, 189, 4, 49, "Subsection"],
Cell[CellGroupData[{
Cell[70296, 2131, 2492, 62, 192, "Input"],
Cell[CellGroupData[{
Cell[72813, 2197, 287, 4, 23, "Print"],
Cell[73103, 2203, 277, 4, 23, "Print"],
Cell[73383, 2209, 297, 4, 23, "Print"],
Cell[73683, 2215, 296, 4, 23, "Print"],
Cell[73982, 2221, 278, 4, 23, "Print"],
Cell[74263, 2227, 292, 4, 23, "Print"],
Cell[74558, 2233, 287, 4, 23, "Print"],
Cell[74848, 2239, 284, 4, 23, "Print"],
Cell[75135, 2245, 392, 6, 23, "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[75588, 2258, 188, 4, 49, "Subsection"],
Cell[CellGroupData[{
Cell[75801, 2266, 2635, 64, 192, "Input"],
Cell[CellGroupData[{
Cell[78461, 2334, 127, 1, 23, "Print"],
Cell[78591, 2337, 145, 1, 23, "Print"],
Cell[78739, 2340, 144, 1, 23, "Print"],
Cell[78886, 2343, 128, 1, 23, "Print"],
Cell[79017, 2346, 140, 1, 23, "Print"],
Cell[79160, 2349, 135, 1, 23, "Print"],
Cell[79298, 2352, 136, 1, 23, "Print"],
Cell[79437, 2355, 242, 3, 23, "Print"]
}, Open  ]]
}, Closed]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[79752, 2366, 235, 3, 86, "Section"],
Cell[79990, 2371, 315, 6, 49, "Text"],
Cell[CellGroupData[{
Cell[80330, 2381, 116, 1, 49, "Subsection"],
Cell[CellGroupData[{
Cell[80471, 2386, 3346, 77, 147, "Input"],
Cell[83820, 2465, 3489, 98, 219, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[87346, 2568, 318, 7, 31, "Input"],
Cell[87667, 2577, 2069, 55, 106, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[89773, 2637, 645, 15, 31, "Input"],
Cell[90421, 2654, 1916, 53, 106, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[92374, 2712, 1394, 35, 67, "Input"],
Cell[93771, 2749, 1013, 25, 61, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[94821, 2779, 1564, 40, 67, "Input"],
Cell[96388, 2821, 1182, 31, 62, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[97607, 2857, 1352, 35, 67, "Input"],
Cell[98962, 2894, 367, 10, 75, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[99366, 2909, 1399, 38, 67, "Input"],
Cell[100768, 2949, 1041, 29, 61, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[101846, 2983, 1839, 50, 107, "Input"],
Cell[103688, 3035, 1426, 40, 80, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[105151, 3080, 2026, 51, 107, "Input"],
Cell[107180, 3133, 1053, 27, 61, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[108270, 3165, 94, 1, 31, "Input"],
Cell[108367, 3168, 713, 21, 61, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[109117, 3194, 93, 1, 39, "Subsubsection"],
Cell[CellGroupData[{
Cell[109235, 3199, 119, 2, 31, "Input"],
Cell[109357, 3203, 871, 11, 404, "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[110289, 3221, 181, 2, 49, "Subsection"],
Cell[CellGroupData[{
Cell[110495, 3227, 2758, 67, 192, "Input"],
Cell[CellGroupData[{
Cell[113278, 3298, 287, 4, 23, "Print"],
Cell[113568, 3304, 277, 4, 23, "Print"],
Cell[113848, 3310, 297, 4, 23, "Print"],
Cell[114148, 3316, 296, 4, 23, "Print"],
Cell[114447, 3322, 278, 4, 23, "Print"],
Cell[114728, 3328, 292, 4, 23, "Print"],
Cell[115023, 3334, 287, 4, 23, "Print"],
Cell[115313, 3340, 284, 4, 23, "Print"],
Cell[115600, 3346, 392, 6, 23, "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[116053, 3359, 180, 2, 49, "Subsection"],
Cell[CellGroupData[{
Cell[116258, 3365, 2824, 67, 192, "Input"],
Cell[CellGroupData[{
Cell[119107, 3436, 127, 1, 23, "Print"],
Cell[119237, 3439, 145, 1, 23, "Print"],
Cell[119385, 3442, 144, 1, 23, "Print"],
Cell[119532, 3445, 128, 1, 23, "Print"],
Cell[119663, 3448, 140, 1, 23, "Print"],
Cell[119806, 3451, 135, 1, 23, "Print"],
Cell[119944, 3454, 136, 1, 23, "Print"],
Cell[120083, 3457, 242, 3, 23, "Print"]
}, Open  ]]
}, Closed]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
